#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import os.path as path
import numpy as np
import tempfile
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
import kapture.io.features as binary


class TestKeypoints(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._temp_filepath = path.join(self._tempdir.name, '000.jpg.kpt')

    def tearDown(self):
        self._tempdir.cleanup()

    def test_write_uint8(self):
        nb_keypoints = 5
        dim_keypoint = 4
        the_answer = 42
        type_keypoint = np.uint8
        image_keypoints = (np.ones((nb_keypoints, dim_keypoint)) * the_answer).astype(type_keypoint)
        binary.array_to_file(self._temp_filepath, image_keypoints)
        with open(self._temp_filepath, 'rt') as file:
            line = file.readline()
        self.assertEqual('********************', line)

    def test_write_read(self):
        nb_keypoints = 5
        dim_keypoint = 4
        type_keypoint = float
        image_keypoints = np.random.random((nb_keypoints, dim_keypoint)).astype(type_keypoint)
        binary.image_keypoints_to_file(self._temp_filepath, image_keypoints)
        image_keypoints_read = binary.image_keypoints_from_file(self._temp_filepath,
                                                                dtype=type_keypoint,
                                                                dsize=dim_keypoint)
        self.assertEqual(image_keypoints.shape, image_keypoints_read.shape)
        self.assertEqual(image_keypoints.dtype, image_keypoints_read.dtype)
        self.assertAlmostEqual(image_keypoints.tolist(), image_keypoints_read.tolist())

    def test_feature_type_none(self):
        self.assertRaises(AssertionError, binary.get_features_fullpath, kapture.Keypoints, None, '')


class TestMatches(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._samples_dirpath = path.abspath(path.join(path.dirname(__file__), '..', 'samples', 'maupertuis'))
        self._kapture_dirpath = path.join(self._samples_dirpath, 'kapture')
        # self._matches_dirpath = path.join(self._kapture_dirpath, 'reconstruction', 'matches')

    def tearDown(self):
        self._tempdir.cleanup()

    def test_matches_from_dir(self):
        self._samples_dirpath
        image_pairs_expected = {('00.jpg', '01.jpg'), ('00.jpg', '02.jpg'),
                                ('00.jpg', '03.jpg'), ('01.jpg', '02.jpg'),
                                ('01.jpg', '03.jpg'), ('02.jpg', '03.jpg')}
        image_pairs_actual = set(kapture.io.features.matching_pairs_from_dirpath('SIFT', self._kapture_dirpath))
        self.assertEqual(6, len(image_pairs_actual))
        self.assertEqual(image_pairs_expected, image_pairs_actual)

        # test matches constructor
        matches = kapture.Matches(image_pairs_expected)
        self.assertEqual(6, len(matches))
        self.assertEqual(image_pairs_expected, matches)

    def test_matches_from_file(self):
        # check a single sample file
        matching_sample_filepath = path.join(self._kapture_dirpath, 'reconstruction', 'matches', 'SIFT',
                                             '00.jpg.overlapping', '01.jpg.matches')
        matchings = kapture.io.features.image_matches_from_file(matching_sample_filepath)
        self.assertIsInstance(matchings, np.ndarray)
        self.assertEqual((2501, 3), matchings.shape)
        self.assertEqual(np.float64, matchings.dtype)
        self.assertAlmostEqual([3., 1., 0.], matchings[0].tolist())

    def test_matches_to_file(self):
        # check a single sample file
        matchings_sample_filepath = path.join(self._tempdir.name, 'dummy.matches')
        matchings = np.random.uniform(0, 10, (100, 3)).astype(np.float64)
        kapture.io.features.image_matches_to_file(matchings_sample_filepath, matchings)
        matchings_retrieved = kapture.io.features.image_matches_from_file(matchings_sample_filepath)
        self.assertAlmostEqual(matchings.tolist(), matchings_retrieved.tolist())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import logging
import sys
import os
import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
from kapture.algo.compare import equal_kapture, equal_sensors, equal_records_gnss
import kapture.io.csv as csv
from kapture.io.records import TransferAction
from kapture.converter.opensfm.import_opensfm import import_opensfm

logger = logging.getLogger('opensfm')


class TestOpenSfM(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup variables for all the tests
        """
        logger.setLevel(logging.CRITICAL)
        cls.isWindows = sys.platform.startswith("win") or sys.platform.startswith("cygwin")
        test_file_path = path.dirname(__file__)
        cls._opensfm_sample_path = path.abspath(path.join(test_file_path, '../samples/berlin/opensfm'))
        cls._kapture_sample_path = path.abspath(path.join(test_file_path, '../samples/berlin/kapture'))

    def setUp(self) -> None:
        """
        Setup before every test
        """
        self._tempdir = tempfile.TemporaryDirectory()
        self._kapture_rebuilt_path = path.join(self._tempdir.name, 'kapture_rebuilt')
        os.makedirs(self._kapture_rebuilt_path, exist_ok=True)

    def test_import_opensfm(self) -> None:
        """
        Test the import_opensfm function on small sample
        """
        # convert and then load
        file_operation = TransferAction.skip if self.isWindows else TransferAction.link_relative
        import_opensfm(
            self._opensfm_sample_path,
            self._kapture_rebuilt_path,
            force_overwrite_existing=False,
            images_import_method=file_operation
        )
        kapture_data_expected = csv.kapture_from_dir(self._kapture_sample_path)
        kapture_data_actual = csv.kapture_from_dir(self._kapture_rebuilt_path)
        # check in detail what could be wrong
        self.assertTrue(equal_sensors(kapture_data_expected.sensors, kapture_data_actual.sensors))
        self.assertTrue(equal_records_gnss(kapture_data_expected.records_gnss, kapture_data_actual.records_gnss))
        # check all at once
        self.assertTrue(equal_kapture(kapture_data_expected, kapture_data_actual))

    def test_export_opensfm(self) -> None:
        """
        Test the import_openmvg function on a small JSON file while linking the images
        """
        pass

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        self._tempdir.cleanup()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

"""
Tests of various rosbag to kapture converters.
"""

import os
import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.algo.compare import equal_kapture
from kapture.core.Sensors import Camera, CameraType
import kapture.io.csv as kcsv
# tools
from kapture.converter.ros_tools.import_utbm_sensor import BB2_CAMERA_IDENTIFIERS, TOPICS_BB2
from kapture.converter.ros_tools.import_utbm_sensor import import_utbm_sensors
from kapture.utils.open_cv import import_opencv_camera_calibration
try:
    import rosbag  # noqa: F401
    from kapture.converter.ros_tools.import_rosbag import RosBagImporter
    has_rosbag = True
except ModuleNotFoundError:
    has_rosbag = False


TOPIC_ODOMETRY = '/camera/odom/sample'


@unittest.skipIf(not has_rosbag, "rosbag module is missing")
class TestImportT265Rosbag(unittest.TestCase):

    def setUp(self) -> None:
        """
        Setup before every test
        """
        samples_t265_folder = path.abspath(path.join(path.dirname(__file__), '../samples/t265'))
        self.bag_file_path = path.join(samples_t265_folder, 'trimmed_locoffice.bag')
        t265_rig_kapture = path.join(samples_t265_folder, 'rigs_only_kapture')
        kapture_data = kcsv.kapture_from_dir(t265_rig_kapture)
        self.ros_sample_kapture_path = path.join(samples_t265_folder, 'ros_kapture')
        self.tempdir = tempfile.TemporaryDirectory()
        self.kapture_path = path.join(self.tempdir.name, 'from_ros')
        self.importer = RosBagImporter(rosbag_path=self.bag_file_path,
                                       rigs=kapture_data.rigs,
                                       sensors=kapture_data.sensors,
                                       kapture_path=self.kapture_path,
                                       force_overwrite_existing=True)
        self.camera_ids = sorted(list(kapture_data.cameras.keys()))
        self.first_cam_id = self.camera_ids[0]

    def tearDown(self) -> None:
        self._tempdir.cleanup()

    def testMissingTopic(self):
        with self.assertRaisesRegex(ValueError, 'Missing image topic',
                                    msg="Missing image topic detected"):
            self.importer.import_multi_camera(odometry_topic="/message/odometry",
                                              image_topics=[],
                                              camera_identifiers=[],
                                              percent=100)
        with self.assertRaisesRegex(ValueError, 'Unequal number of .*',
                                    msg="Missing camera identifier detected"):
            self.importer.import_multi_camera(odometry_topic="/message/odometry",
                                              image_topics="/camera/image/left",
                                              camera_identifiers=[],
                                              percent=100)

    def testUnknownTopic(self):
        with self.assertRaisesRegex(ValueError, 'Missing topic .* Rosbag',
                                    msg="Unknown odometry topic detected"):
            self.importer.import_multi_camera(odometry_topic="/message/odometry",
                                              image_topics="/camera/image/left",
                                              camera_identifiers=self.first_cam_id,
                                              percent=100)
        with self.assertRaisesRegex(ValueError, 'Missing image topic .* Rosbag',
                                    msg="Unknown image topic detected"):
            self.importer.import_multi_camera(odometry_topic=TOPIC_ODOMETRY,
                                              image_topics="/camera/image/left",
                                              camera_identifiers=self.first_cam_id,
                                              percent=100)

    def testInvalidCameraIdentifiers(self):
        with self.assertRaisesRegex(ValueError, 'Camera identifier left .* not defined',
                                    msg="Invalid camera identifier"):
            self.importer.import_multi_camera(odometry_topic=TOPIC_ODOMETRY,
                                              image_topics=["/camera/image/left"],
                                              camera_identifiers=["left"],
                                              percent=100)

    def test_import_t265_rosbag(self):
        # rosbag was trimmed with
        # filter LocOffice_2019-03-21-15-57-04.bag trimmed_bag.bag "t.secs <= 1553180224 and t.nsecs <= 460916281"
        self.importer.import_multi_camera(odometry_topic=TOPIC_ODOMETRY,
                                          image_topics=['/camera/fisheye1/image_raw', '/camera/fisheye2/image_raw'],
                                          camera_identifiers=self.camera_ids,
                                          save_all_positions=False,
                                          find_image_position=True)
        rig_id = list(self.importer._rigs.keys())[0]
        self.importer.save_to_kapture(rig_id)

        rosbag_kapture_data = kcsv.kapture_from_dir(self.ros_sample_kapture_path)
        imported_data = kcsv.kapture_from_dir(self.kapture_path)
        self.assertEqual(len(imported_data.trajectories), len(rosbag_kapture_data.records_camera),
                         "one pose per image pair")
        self.assertTrue(equal_kapture(rosbag_kapture_data, imported_data), "Imported kapture ok")

        images_path = kapture.io.records.get_image_fullpath(self.kapture_path)
        images_files = []
        for root, dirs, files in os.walk(images_path):
            for name in files:
                images_files.append(path.join(root, name))
        self.assertEqual(len(images_files), 6)
        # Check the files exist
        for image_file in images_files:
            self.assertTrue(path.isfile(image_file), f"Image file {image_file} exist")

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        self.tempdir.cleanup()


class TestImportUtbmRosbag(unittest.TestCase):

    def setUp(self) -> None:
        """
        Setup before every test
        """
        self._samples_utbm_folder = path.abspath(path.join(path.dirname(__file__), '../samples/utbm'))
        self.tempdir = tempfile.TemporaryDirectory()
        self.kapture_path = path.join(self.tempdir.name, 'from_ros')

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        self.tempdir.cleanup()

    def test_read_bb2_camera_info(self) -> None:
        """
        Test the creation of a kapture camera object from a camera info file
        """
        cam_info_file = path.join(self._samples_utbm_folder, 'bb2.yaml')
        sensors = import_utbm_sensors(cam_info_file)
        self.assertEqual(1, len(sensors), "Created one sensor")
        camera_name = list(sensors)[0]
        self.assertEqual('bb2_cam', camera_name, "Correct camera name")
        camera = sensors[camera_name]
        self.assertIsInstance(camera, Camera, "Is of type Camera")
        self.assertEqual(CameraType.OPENCV, camera.camera_type, "of type openCV")
        self.assertEqual('bb2_cam', camera.name, "Named bb2_cam")
        self.assertEqual(1024, camera.camera_params[0], "Image width")
        self.assertEqual(768, camera.camera_params[1], "Image height")

    def test_read_bb2_with_k3_camera_info(self) -> None:
        """
        Test exception thrown when a camera info file k3 parameter is not zero
        """
        cam_info_file = path.join(self._samples_utbm_folder, 'bb2_with_k3.yaml')
        camera = import_opencv_camera_calibration(cam_info_file)
        self.assertIsInstance(camera, Camera, "Is of type Camera")
        self.assertEqual(CameraType.FULL_OPENCV, camera.camera_type, "of type full openCV")
        self.assertEqual('bb2_cam', camera.name, "Named bb2_cam")
        self.assertEqual(1024, camera.camera_params[0], "Image width")
        self.assertEqual(768, camera.camera_params[1], "Image height")
        self.assertNotEqual(0.0, camera.camera_params[10], "K3 is not null")

    @unittest.skipIf(not has_rosbag, "rosbag module is missing")
    def test_utbm_images_rosbag_import(self) -> None:
        """
        Test the import of an image rosbag
        """
        # Use a small bb2 rosbag
        rosbag_path = path.join(self._samples_utbm_folder, '2018-07-13-15-17-20_1_first10_bb2.bag')
        sensors = import_utbm_sensors([path.join(self._samples_utbm_folder, 'bb2_left.yaml'),
                                       path.join(self._samples_utbm_folder, 'bb2_right.yaml')])
        importer = RosBagImporter(rosbag_path, None, sensors, self.kapture_path, force_overwrite_existing=True)
        importer.import_multi_camera(None, TOPICS_BB2, BB2_CAMERA_IDENTIFIERS, True, False, False)
        importer.save_to_kapture()
        ros_sample_kapture_path = path.join(self._samples_utbm_folder, '2018-07-13-15-17-20_1_first10_bb2_kapture')
        rosbag_kapture_data = kcsv.kapture_from_dir(ros_sample_kapture_path)
        imported_data = kcsv.kapture_from_dir(self.kapture_path)
        self.assertTrue(equal_kapture(rosbag_kapture_data, imported_data), "Imported UTBM kapture ok")

        images_path = kapture.io.records.get_image_fullpath(self.kapture_path)
        images_files = []
        for root, dirs, files in os.walk(images_path):
            for name in files:
                images_files.append(path.join(root, name))
        self.assertEqual(len(images_files), 10)
        # Check the files exist
        for image_file in images_files:
            self.assertTrue(path.isfile(image_file), f"Image file {image_file} exist")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import tempfile
import unittest
import numpy as np
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.converter.nmea.import_nmea import extract_gnss_from_nmea


class TestImportNmea(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        test_dir_path = path.dirname(__file__)
        cls._samples_folder = path.abspath(path.join(test_dir_path, '../samples/4seasons'))
        cls._tmu_sample_path = path.abspath(path.join(cls._samples_folder, 'TUM'))
        cls._kapture_sample_path = path.abspath(path.join(cls._samples_folder, 'kapture'))
        cls._tempdir = tempfile.TemporaryDirectory()

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        self._tempdir.cleanup()

    def test_read_nmea(self):
        nmea_file_path = path.join(self._tmu_sample_path, 'recording_2020-10-07_14-47-51', 'septentrio.nmea')
        gnss_id = 'my_gps'
        kapture_sensors, records_gnss = extract_gnss_from_nmea(nmea_file_path, gnss_id)

        # check sensor part
        self.assertIsInstance(kapture_sensors, kapture.Sensors)
        self.assertEqual(len(kapture_sensors), 1)
        self.assertIn(gnss_id, kapture_sensors)
        gnss_sensor = kapture_sensors[gnss_id]
        self.assertEqual(gnss_sensor.sensor_type, kapture.SensorType.gnss.name)
        self.assertEqual(gnss_sensor.sensor_params[0], 'EPSG:4326')
        self.assertEqual(gnss_sensor.name, gnss_id)

        # check records
        self.assertIsInstance(records_gnss, kapture.RecordsGnss)
        self.assertEqual(len(records_gnss), 3413)
        timestamps = np.array([int(k) for k in records_gnss.keys()])
        timestamp_range = np.min(timestamps), np.max(timestamps)
        self.assertEqual(timestamp_range, (1602074797700000000, 1602075208800000000))
        first_records = records_gnss[1602074797700000000]
        self.assertIn(gnss_id, first_records)
        first_record = first_records[gnss_id]
        first_record_expected = kapture.RecordGnss(x=11.619441895, y=48.19602170666667, z=494.6412,
                                                   utc=1602074797700000000, dop=2.6)
        self.assertEqual(first_record, first_record_expected)



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import tempfile
import shutil
import unittest
import piexif

# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.records import images_to_filepaths, get_image_fullpath
from kapture.converter.exif.import_exif import convert_gps_to_kapture_record, extract_gps_from_exif, \
    import_gps_from_exif, read_exif
from kapture.converter.exif.export_exif import export_gps_to_exif, write_exif, clear_exif

from kapture.algo.compare import equal_kapture, equal_records_gnss


class TestImportExif(unittest.TestCase):

    def setUp(self):
        self._samples_folder = path.abspath(path.join(path.dirname(__file__), '../samples/'))
        self._kapture_dirpath = path.join(self._samples_folder, 'berlin', 'kapture')
        self._kapture_data = kapture.io.csv.kapture_from_dir(self._kapture_dirpath)
        self._tempdir = tempfile.TemporaryDirectory()

    def tearDown(self):
        self._tempdir.cleanup()

    def test_read_exif_invalid(self):
        invalid_image_filepath = path.join(self._samples_folder,
                                           '7scenes/microsoft/stairs/seq-01/frame-000000.color.jpg')
        exif_data = read_exif(invalid_image_filepath)
        self.assertIsNone(exif_data)

    def test_read_exif(self):
        image_filepath_w_exif = path.join(self._samples_folder, 'berlin/opensfm/images/01.jpg')
        exif_data = read_exif(image_filepath_w_exif)
        self.assertIsInstance(exif_data, dict)
        self.assertEqual(6, len(exif_data))
        self.assertIn('GPS', exif_data)

    def test_read_exif_gps(self):
        image_filepath_w_exif = path.join(self._samples_folder, 'berlin/opensfm/images/01.jpg')
        exif_data = read_exif(image_filepath_w_exif)
        self.assertIn('GPS', exif_data)
        gps_data = exif_data['GPS']
        self.assertIsInstance(gps_data, dict)
        self.assertEqual(6, len(gps_data))
        self.assertIn(piexif.GPSIFD.GPSLatitude, gps_data)

    def test_extract(self):
        sensors_gnss, records_gnss = extract_gps_from_exif(self._kapture_data, self._kapture_dirpath)
        # check sensors
        gps_id = 'GPS_v2 apple iphone 4s back camera 4.28mm f/2.4 3264 2448 perspective 0.9722'
        self.assertEqual(1, len(sensors_gnss))
        self.assertTrue(all(gnss_id.startswith('GPS_') for gnss_id in sensors_gnss.keys()))
        self.assertIn(gps_id, sensors_gnss)
        sensor_gnss = sensors_gnss[gps_id]
        self.assertEqual(kapture.SensorType.gnss.name, sensor_gnss.sensor_type)
        self.assertEqual(['EPSG:4326'], sensor_gnss.sensor_params)
        # check gps track
        self.assertEqual(3, len(records_gnss))
        first_pos = records_gnss[0, gps_id]
        expected_pos = kapture.RecordGnss(13.400294444444445, 52.51891666666666, 27.0, 0, 5.0)
        self.assertEqual(first_pos, expected_pos)

    def test_import(self):
        temp_kapture_dirpath = path.join(self._tempdir.name, 'kapture')
        shutil.copytree(self._kapture_dirpath, temp_kapture_dirpath)
        # remove all GPS data
        temp_kapture_data = kapture.io.csv.kapture_from_dir(temp_kapture_dirpath)
        temp_kapture_data.records_gnss = None
        for gps_id in [k for k in temp_kapture_data.sensors.keys() if k.startswith('GPS_')]:
            del temp_kapture_data.sensors[gps_id]
        kapture.io.csv.kapture_to_dir(temp_kapture_dirpath, temp_kapture_data)
        # do the import
        import_gps_from_exif(temp_kapture_dirpath)
        # reload
        temp_kapture_data = kapture.io.csv.kapture_from_dir(temp_kapture_dirpath)
        self.assertTrue(equal_kapture(temp_kapture_data, self._kapture_data))

    def test_write_exif(self):
        image_filepath_wo_exif = path.join(self._samples_folder,
                                           '7scenes/microsoft/stairs/seq-01/frame-000000.color.jpg')
        image_filepath_w_exif = path.join(self._samples_folder, 'berlin/opensfm/images/01.jpg')
        image_filepath_temp = path.join(self._tempdir.name, 'frame-000000.color.jpg')
        shutil.copy(image_filepath_wo_exif, image_filepath_temp)
        expected_exif = read_exif(image_filepath_w_exif)
        write_exif(image_filepath_temp, expected_exif)
        actual_exif = read_exif(image_filepath_temp)
        # self.assertEqual(len(expected_exif), len(actual_exif))
        self.assertEqual(set(expected_exif.keys()), set(actual_exif.keys()))
        self.assertDictEqual(expected_exif['GPS'], actual_exif['GPS'])

    def test_clear_exif(self):
        image_filepath_w_exif = path.join(self._samples_folder, 'berlin/opensfm/images/01.jpg')
        image_filepath_temp = path.join(self._tempdir.name, 'image.jpg')
        shutil.copy(image_filepath_w_exif, image_filepath_temp)
        clear_exif(image_filepath_temp)
        exif_data = read_exif(image_filepath_temp)
        self.assertIsNone(exif_data)

    def test_export(self):
        temp_kapture_dirpath = path.join(self._tempdir.name, 'kapture')
        shutil.copytree(self._kapture_dirpath, temp_kapture_dirpath)
        kapture_data = kapture.io.csv.kapture_from_dir(temp_kapture_dirpath)
        images_filepaths = images_to_filepaths(kapture_data.records_camera, temp_kapture_dirpath)
        # make sure there is no EXIF in images
        for image_filepath in images_filepaths.values():
            clear_exif(image_filepath)

        # insert gps to exif
        export_gps_to_exif(kapture_data, temp_kapture_dirpath)

        rebuilt_records = kapture.RecordsGnss()
        for timestamp, cam_id, image_name in kapture.flatten(kapture_data.records_camera):
            image_filepath = get_image_fullpath(temp_kapture_dirpath, image_name)
            exif_data = read_exif(image_filepath)
            rebuilt_records[timestamp, 'GPS_' + cam_id] = convert_gps_to_kapture_record(exif_data)

        self.assertTrue(equal_records_gnss(kapture_data.records_gnss, rebuilt_records))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

"""
Test some of the utils functions and classes.
"""

import datetime
import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
import kapture.utils.computation as computation
import kapture.utils.paths


class TestUtils(unittest.TestCase):

    def setUp(self) -> None:
        """
        Setup before every test
        """
        pass

    def testRemoveFileForce(self):
        # Creates a random file
        one_file = tempfile.NamedTemporaryFile().name
        with open(one_file, 'w') as f_dst:
            f_dst.write('some content')
        self.assertTrue(path.isfile(one_file), "Test file created")
        # Remove it
        kapture.utils.paths.safe_remove_file(one_file, True)
        self.assertFalse(path.isfile(one_file), "Test file successfully removed")

    def testPrependToFile(self):
        # Creates a random file
        one_file = tempfile.NamedTemporaryFile(suffix='.txt').name
        with open(one_file, 'w') as f_dst:
            f_dst.write('some content')
        self.assertTrue(path.isfile(one_file), "Test file created")
        additional = 'new first line\n'
        kapture.utils.paths.prepend_to_file(one_file, additional)
        with open(one_file, 'r') as f:
            first_line = f.readline()
            self.assertEqual(additional, first_line, "Successful prepend")

    def testDigitsNumNotAnInt(self):
        now = datetime.datetime.now().timestamp()
        self.assertRaises(TypeError, computation.num_digits, now, msg="must be called with an integer")

    def testDigitsZero(self):
        number = 0
        self.assertEqual(computation.num_digits(number), 1, "1 digit")

    def testDigitsNum(self):
        number = 1234567890
        self.assertEqual(computation.num_digits(number), 10, "10 digits")

    def testDigitsNumEpoch(self):
        # Computes number of digits of typical epoch in microseconds
        now = int(datetime.datetime.now().timestamp()*1000000)
        self.assertEqual(computation.num_digits(now), 16, "Epoch now has 16 digits")

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        pass


if __name__ == '__main__':
    unittest.main()



# Copyright 2020-present NAVER Corp. Under BSD 3-clause license
import logging
from tqdm import tqdm
import os
import os.path as path
from random import randint
from typing import Dict
import numpy as np

import kapture
from .csv import kapture_linesep
from .features import image_keypoints_from_file


logger = logging.getLogger('ply')


PLY_HEADER_TEMPLATE = kapture_linesep.join([
    'ply',
    'format ascii 1.0',
    'element vertex {nb_vertex}',
    'property double x',
    'property double y',
    'property double z',
    'property uchar red',
    'property uchar green',
    'property uchar blue',
    'element edge {nb_edges}',
    'property int vertex1',
    'property int vertex2',
    'end_header'])

########################################################################################################################
BLACK = 3 * [0]
WHITE = 3 * [255]
GREY = 3 * [127]
RED = [255, 0, 0]
GREEN = [0, 255, 0]
BLUE = [0, 0, 255]
AXIS_COLORS = [GREY, RED, GREEN, BLUE]


########################################################################################################################
def get_axis_in_world(
        pose_device_from_world: kapture.PoseTransform, length: float = 1.0) -> kapture.Points3d:
    """
    Returns a quadruplet of points (0,x,y,z) representing the axis of the device into the world.

    :param pose_device_from_world: assume the transformation is device from world.
    :param length: distance between each axis point and center.
    :return: 4 points (center, x, y, z) arranged by cols
    """
    assert (isinstance(pose_device_from_world, kapture.PoseTransform))
    sensor_axis = np.array([
        [0, 0, 0],  # 0
        [length, 0, 0],  # X
        [0, length, 0],  # Y
        [0, 0, length],  # Z
    ], dtype=np.float)
    return pose_device_from_world.inverse().transform_points(sensor_axis)


def header_to_ply_stream(stream, nb_vertex: int = 0, nb_edges: int = 0) -> None:
    """
    Writes PLY header to a stream.

    :param stream: an open stream to write to
    :param nb_vertex: number of vertex
    :param nb_edges: number of edges
    """
    stream.write(PLY_HEADER_TEMPLATE.format(nb_vertex=nb_vertex,
                                            nb_edges=nb_edges))
    stream.write(kapture_linesep)


def rig_to_ply_stream(stream, rig: Dict[str, kapture.PoseTransform], axis_length: float = 1.) -> None:
    """
    Writes the rig to a stream.

    :param stream: an open stream to write to
    :param rig: rig to write
    :param axis_length: length of the axis
    """
    device_list = [(cam_id, pose_tr)
                   for cam_id, pose_tr in rig.items()]

    # add the rig center in devices
    device_list = [(-1, kapture.PoseTransform())] + device_list

    # create 4 points per device: 1 for center, 3 for axis
    points_colored_list = []
    edges_list = []
    for cam_id, pose_tr in device_list:
        axis = get_axis_in_world(pose_tr, axis_length)
        edges_list += [(0, len(points_colored_list))]
        points_colored_list += [p + AXIS_COLORS[i] for i, p in enumerate(axis.tolist())]

    # write points into ply
    header_to_ply_stream(stream,
                         nb_vertex=len(points_colored_list),
                         nb_edges=len(edges_list))
    for p3d in points_colored_list:
        line = ['{:<25}'.format(i) for i in p3d[0:3]]
        line += ['{:03}'.format(int(i)) for i in p3d[3:6]]
        stream.write(' '.join(line) + kapture_linesep)
    for e in edges_list:
        line = ['{:2}'.format(i) for i in e]
        stream.write(' '.join(line) + kapture_linesep)


def rig_to_ply(filepath: str, rig: Dict[str, kapture.PoseTransform], axis_length: float = 1.) -> None:
    """
    Writes the rig to a file.

    :param filepath: file path to write to
    :param rig: rig to write
    :param axis_length: length of the axis
    """
    with open(filepath, 'w') as f:
        rig_to_ply_stream(f, rig, axis_length)


########################################################################################################################
def trajectories_to_ply_stream(stream, trajectories: kapture.Trajectories, axis_length: float = 1.) -> None:
    """
    Writes the trajectories to a stream.
     trajectories[ts][device_id] = [pose]

    :param stream: an open stream to write to
    :param trajectories: trajectories to write
    :param axis_length: length of the axis
    """
    pose_list = (pose_tr
                 for _, _, pose_tr in kapture.flatten(trajectories, is_sorted=True)
                 if not np.any(np.isnan(pose_tr.t)))  # filter out if no position

    # create 4 points per pose: 1 for center, 3 for axis
    points_colored_list = []
    for pose_tr in pose_list:
        axis = get_axis_in_world(pose_device_from_world=pose_tr, length=axis_length)
        points_colored_list += [p + AXIS_COLORS[i] for i, p in enumerate(axis.tolist())]

    # write points into ply
    header_to_ply_stream(stream, nb_vertex=len(points_colored_list))
    hide = logger.getEffectiveLevel() >= logging.CRITICAL
    for p3d in tqdm(points_colored_list, disable=hide):
        line = ['{:<25}'.format(i) for i in p3d[0:3]]
        line += ['{:<4}'.format(i) for i in p3d[3:6]]
        stream.write(' '.join(line) + kapture_linesep)


def trajectories_to_ply(
        filepath: str,
        trajectories: kapture.Trajectories,
        axis_length: float = 1.
):
    """
    Writes trajectory to PLY format (for visualization).
    Each pose in trajectory leads to a ply dot. 3 additional points are added in X (red), Y (Green) and Z (blue)
    direction around each pose.

    :param filepath: input ply file path.
    :param trajectories: input trajectory
    :param axis_length: length of axis representing the orientation of each pose in trajectory.
    :return:
    """
    os.makedirs(path.dirname(filepath), exist_ok=True)
    with open(filepath, 'w') as fout:
        trajectories_to_ply_stream(stream=fout, trajectories=trajectories, axis_length=axis_length)


def points3d_to_stream(stream, points3d: kapture.Points3d) -> None:
    """
    Writes the 3D points to a stream.

    :param stream: an open stream to write to
    :param points3d: 3d points to write
    """
    header_to_ply_stream(stream, nb_vertex=len(points3d))
    hide = logger.getEffectiveLevel() >= logging.CRITICAL
    for p3d in tqdm(points3d, disable=hide):
        line = ['{:20}'.format(i) for i in p3d[0:3]] + ['{:03d}'.format(int(i)) for i in p3d[3:6]]
        stream.write('  '.join(line) + kapture_linesep)


def points3d_to_ply(filepath: str, points3d: kapture.Points3d) -> None:
    """
    Writes 3D points into ply file.

    :param filepath: ply file path.
    :param points3d: 3D points.
    """
    os.makedirs(path.dirname(filepath), exist_ok=True)
    with open(filepath, 'w') as f:
        points3d_to_stream(f, points3d)


def local_points3d_to_stream(
        stream,
        points3d: np.ndarray,
        transform_world_from_local: kapture.PoseTransform
) -> None:
    """
    Writes the 3D points from a local coordinate system into world into a stream.

    :param stream: an open stream to write to
    :param points3d: input 3d points as a Nx3 numpy array
    :param transform_world_from_local: transformation
    """
    # sanity check
    if not isinstance(points3d, np.ndarray) or points3d.shape[1] != 3:
        raise TypeError('expect 3d points as a numpy array.')

    header_to_ply_stream(stream, nb_vertex=points3d.shape[0])
    for p3d_local in points3d:
        p3d_world = transform_world_from_local.transform_points(p3d_local)
        line = ['{:20}'.format(i) for i in p3d_world]
        stream.write('  '.join(line) + kapture_linesep)


def local_points3d_to_ply(
        filepath: str,
        points3d: np.ndarray,
        transform_world_from_local: kapture.PoseTransform
) -> None:
    """
    Writes 3D points into ply file.

    :param filepath: ply file path.
    :param points3d: input 3d points as a Nx3 numpy array
    :param transform_world_from_local: transformation
    """
    os.makedirs(path.dirname(filepath), exist_ok=True)
    with open(filepath, 'w') as f:
        local_points3d_to_stream(f, points3d, transform_world_from_local)


def image_keypoints_to_stream(stream, image_keypoints: np.array) -> None:
    """
    Plots image keypoints onto a 2D plane. Use random colors.

    :param stream: an open stream to write to
    :param image_keypoints: the image keypoints to write
    """
    header_to_ply_stream(stream, nb_vertex=len(image_keypoints))
    for kpt in image_keypoints:
        coords = kpt[0:2]
        line = ['{:20}'.format(i) for i in coords] + ['0.0'] + ['{:03d}'.format(randint(0, 255)) for _ in range(3)]
        stream.write('  '.join(line) + kapture_linesep)


def image_keypoints_to_ply(ply_filepath: str, image_keypoints_filepath: str, keypoint_dtype, keypoint_dsize) -> None:
    """
    Plots image keypoints onto a 2D plane.

    :param ply_filepath: path to the ply file to write
    :param image_keypoints_filepath: path to the image keypoints file to read
    :param keypoint_dtype: keypoint data type
    :param keypoint_dsize: keypoint data size
    """
    os.makedirs(path.dirname(ply_filepath), exist_ok=True)
    image_keypoints = image_keypoints_from_file(image_keypoints_filepath, keypoint_dtype, keypoint_dsize)
    with open(ply_filepath, 'w') as f:
        image_keypoints_to_stream(f, image_keypoints)



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.csv import kapture_from_dir
from kapture.algo.compare import equal_kapture
from tools.kapture_import_image_list import import_image_list
from tools.kapture_export_image_list import export_image_list


class TestImageListAachen(unittest.TestCase):

    def setUp(self):
        samples_folder = path.abspath(path.join(path.dirname(__file__),  '../samples/'))
        self.aachen_folder = path.join(samples_folder, 'Aachen-Day-Night')
        self.aachen_models_folder = path.join(self.aachen_folder, '3D-models')
        self.images_folder = path.join(self.aachen_folder, 'images_upright')
        self.query_folder = path.join(self.aachen_folder, 'queries')
        self.kapture_query_path = path.join(self.aachen_folder, 'kapture/query')

    def test_import_with_intrinsics(self):
        with tempfile.TemporaryDirectory() as tmpdirname:
            queries_with_intrinsics_path = path.join(self.query_folder, 'day_time_queries_with_intrinsics.txt')

            import_image_list([queries_with_intrinsics_path], self.images_folder, tmpdirname,
                              force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(self.kapture_query_path)
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

    def test_import_without_intrinsics(self):
        with tempfile.TemporaryDirectory() as tmpdirname:
            queries_without_intrinsics_path = path.join(self.query_folder, 'day_time_queries_without_intrinsics.txt')

            import_image_list([queries_without_intrinsics_path], self.images_folder, tmpdirname,
                              force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(self.kapture_query_path)
            # set all sensors to unknown
            for sensor_id in expected_kdata.sensors.keys():
                sensor = expected_kdata.sensors[sensor_id]
                assert isinstance(sensor, kapture.Camera)
                expected_kdata.sensors[sensor_id] = kapture.Camera(kapture.CameraType.UNKNOWN_CAMERA,
                                                                   sensor.camera_params[0:2])
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

    def test_export_with_intrinsics(self):
        with tempfile.TemporaryDirectory() as tmpdirname:
            output_filename = path.join(tmpdirname, 'image_list.txt')
            export_image_list(self.kapture_query_path, output_filename, export_camera_params=True, force=True)

            with open(output_filename) as file:
                exported_content = file.readlines()

            queries_with_intrinsics_path = path.join(self.query_folder, 'day_time_queries_with_intrinsics.txt')
            with open(queries_with_intrinsics_path) as file:
                expected_content = file.readlines()

            self.assertListEqual(exported_content, expected_content)

    def test_export_without_intrinsics(self):
        with tempfile.TemporaryDirectory() as tmpdirname:
            output_filename = path.join(tmpdirname, 'image_list.txt')
            export_image_list(self.kapture_query_path, output_filename, export_camera_params=False, force=True)

            with open(output_filename) as file:
                exported_content = file.readlines()

            queries_without_intrinsics_path = path.join(self.query_folder, 'day_time_queries_without_intrinsics.txt')
            with open(queries_without_intrinsics_path) as file:
                expected_content = file.readlines()

            self.assertListEqual(exported_content, expected_content)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.features import FEATURE_FILE_EXTENSION, get_keypoints_fullpath, image_keypoints_from_file
from kapture.algo.compare import equal_kapture, equal_sensors, equal_records_gnss
import kapture.io.csv as csv
import numpy as np


class TestTar(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup variables for all the tests
        """
        test_file_path = path.dirname(__file__)
        cls._kapture_sample_path = path.abspath(path.join(test_file_path, '../samples/berlin/kapture'))
        cls._kapture_tar_sample_path = path.abspath(path.join(test_file_path, '../samples/berlin/kapture_tar'))

    def test_kapture_from_dir(self) -> None:
        """
        Test the kapture_from_dir function on small sample
        """
        # convert and then load
        kapture_data_expected = csv.kapture_from_dir(self._kapture_sample_path)
        kapture_data_no_recon = csv.kapture_from_dir(self._kapture_tar_sample_path)
        # check in detail what could be wrong
        self.assertTrue(equal_sensors(kapture_data_expected.sensors, kapture_data_no_recon.sensors))
        self.assertTrue(equal_records_gnss(kapture_data_expected.records_gnss, kapture_data_no_recon.records_gnss))
        self.assertTrue(kapture_data_no_recon.keypoints is not None
                        and 'HessianAffine' in kapture_data_no_recon.keypoints
                        and len(kapture_data_no_recon.keypoints['HessianAffine']) == 0)
        self.assertTrue(kapture_data_no_recon.descriptors is not None
                        and 'HOG' in kapture_data_no_recon.descriptors
                        and len(kapture_data_no_recon.descriptors['HOG']) == 0)
        self.assertTrue(kapture_data_no_recon.matches is not None
                        and 'HessianAffine' in kapture_data_no_recon.matches
                        and len(kapture_data_no_recon.matches['HessianAffine']) == 0)
        # check all at once
        self.assertFalse(equal_kapture(kapture_data_expected, kapture_data_no_recon))

        with csv.get_all_tar_handlers(self._kapture_tar_sample_path) as tar_handlers:
            kapture_data_actual = csv.kapture_from_dir(self._kapture_tar_sample_path, tar_handlers=tar_handlers)

            # check in detail what could be wrong
            self.assertTrue(equal_sensors(kapture_data_expected.sensors, kapture_data_actual.sensors))
            self.assertTrue(equal_records_gnss(kapture_data_expected.records_gnss, kapture_data_actual.records_gnss))
            # check all at once
            self.assertTrue(equal_kapture(kapture_data_expected, kapture_data_actual))

            # check kpt
            image_name = '01.jpg'
            keypoints_type = 'HessianAffine'
            kapture_keypoints_filepath = get_keypoints_fullpath(keypoints_type,
                                                                kapture_dirpath=self._kapture_sample_path,
                                                                image_filename=image_name)
            kpts_expected = image_keypoints_from_file(kapture_keypoints_filepath,
                                                      kapture_data_expected.keypoints[keypoints_type].dtype,
                                                      kapture_data_expected.keypoints[keypoints_type].dsize)
            handler_kpts = tar_handlers.keypoints[keypoints_type]
            kpts_actual = handler_kpts.get_array_from_tar(image_name + FEATURE_FILE_EXTENSION[kapture.Keypoints],
                                                          kapture_data_expected.keypoints[keypoints_type].dtype,
                                                          kapture_data_expected.keypoints[keypoints_type].dsize)
            self.assertTrue(np.array_equal(kpts_expected, kpts_actual))

            kapture_keypoints_filepath = get_keypoints_fullpath(keypoints_type,
                                                                kapture_dirpath=self._kapture_sample_path,
                                                                image_filename=image_name,
                                                                tar_handler=tar_handlers)
            kpts_expected = image_keypoints_from_file(kapture_keypoints_filepath,
                                                      kapture_data_expected.keypoints[keypoints_type].dtype,
                                                      kapture_data_expected.keypoints[keypoints_type].dsize)
            self.assertTrue(np.array_equal(kpts_expected, kpts_actual))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import os
import tempfile
import tarfile
import os.path as path
import sys
import stat
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture.converter.downloader.archives as archives
# import tools.kapture_download_dataset as download
# from unittest.mock import patch

"""
Test dataset list integrity
Returns error if a dataset is not reachable
"""

SLOW_TESTS = os.environ.get('SLOW_TESTS', False)

#
# class TestDownload(unittest.TestCase):
#
#     def setUp(self):
#         self.dataset_dir = path.abspath(path.join(path.dirname(__file__),  '../dataset'))
#
#     def test_update(self):
#         test_args = ["downloader", "--install_path", self.dataset_dir, "update"]
#         with patch.object(sys, 'argv', test_args):
#             self.assertEqual(download.kapture_download_dataset_cli(), 0)
#
#     @unittest.skipUnless(SLOW_TESTS, "slow test")
#     def test_all_datasets(self):
#         test_args = ["downloader", "--install_path", self.dataset_dir, "list", "--full"]
#         with patch.object(sys, 'argv', test_args):
#             self.assertEqual(download.kapture_download_dataset_cli(), 0)


class TestDownloaderPermissions(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        # make up a read only file
        test_dirname = 'permission_dir'
        test_filename = path.join(test_dirname, 'permission_file.txt')
        self.test_filepath = path.join(self._tempdir.name, test_filename)
        self.test_dirpath = path.join(self._tempdir.name, test_dirname)
        os.makedirs(self.test_dirpath)
        with open(self.test_filepath, 'wt') as f:
            f.write('permission')
        os.chmod(self.test_filepath, stat.S_IRUSR)
        # tar it
        self.tar_filepath = path.join(self._tempdir.name, 'archive.tar')
        with tarfile.open(self.tar_filepath, 'w:gz') as tar:
            tar.add(self.test_filepath, arcname=test_filename)
        # clean
        os.chmod(self.test_filepath, stat.S_IWUSR)
        os.remove(self.test_filepath)
        os.rmdir(self.test_dirpath)

    def tearDown(self) -> None:
        self._tempdir.cleanup()

    def test_extract_permissions(self):
        archives.untar_file(archive_filepath=self.tar_filepath, install_dirpath=self._tempdir.name)
        st = os.stat(self.test_filepath)
        self.assertTrue(st.st_mode & stat.S_IRUSR)
        self.assertTrue(st.st_mode & stat.S_IWUSR)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.csv import kapture_from_dir
from kapture.algo.compare import equal_kapture
from tools.kapture_import_bundler import import_bundler


class TestImportBundler(unittest.TestCase):

    def setUp(self):
        samples_folder = path.abspath(path.join(path.dirname(__file__),  '../samples/'))
        self.aachen_folder = path.join(samples_folder, 'Aachen-Day-Night')
        self.aachen_models_folder = path.join(self.aachen_folder, '3D-models')
        self.images_folder = path.join(self.aachen_folder, 'images_upright')
        self.bundler_sensors = kapture.Sensors()
        self.bundler_sensors['sensor0'] = kapture.Camera(kapture.CameraType.RADIAL,
                                                         [1600, 1067, 1.084590000e+03,
                                                          800, 533.5, 0.000000000e+00, 6.894198313e-08])
        self.bundler_sensors['sensor1'] = kapture.Camera(kapture.CameraType.RADIAL,
                                                         [1200, 1600, 1.556980000e+03,
                                                          600, 800, 0.000000000e+00, 3.565154420e-08])
        self.bundler_sensors['sensor2'] = kapture.Camera(kapture.CameraType.RADIAL,
                                                         [1600, 1067, 1.103400000e+03,
                                                          800, 533.5, 0.000000000e+00, 6.527248534e-08])

    def test_import(self):
        bundler_file_path = path.join(self.aachen_models_folder, 'aachen_cvpr2018_db.out')
        bundler_imagelist_path = path.join(self.aachen_models_folder, 'aachen_cvpr2018_db.list.txt')

        with tempfile.TemporaryDirectory() as tmpdirname:
            import_bundler(bundler_file_path, bundler_imagelist_path, self.images_folder, tmpdirname,
                           ignore_trajectories=False, add_reconstruction=True,
                           force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata.sensors = self.bundler_sensors
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test without trajectories
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_bundler(bundler_file_path, bundler_imagelist_path, self.images_folder, tmpdirname,
                           ignore_trajectories=True, add_reconstruction=True,
                           force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata.sensors = self.bundler_sensors
            expected_kdata._trajectories = None
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test without points3d
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_bundler(bundler_file_path, bundler_imagelist_path, self.images_folder, tmpdirname,
                           ignore_trajectories=False, add_reconstruction=False,
                           force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata.sensors = self.bundler_sensors
            expected_kdata._points3d = None
            expected_kdata._keypoints = None
            expected_kdata._observations = None
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import os.path as path
import tempfile
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
from kapture.io.csv import kapture_from_dir
from kapture.algo.compare import equal_kapture
from kapture.converter.virtual_gallery.import_virtual_gallery import import_virtual_gallery  # noqa: E402
from kapture.core.Trajectories import rigs_remove_inplace
from kapture.io.records import TransferAction


class TestImportVirtualGallery(unittest.TestCase):

    def setUp(self):
        samples_folder = path.abspath(path.join(path.dirname(__file__),  '../samples/'))
        self.virtual_gallery_folder = path.join(samples_folder, 'virtual_gallery')
        self.virtual_gallery_1_0_0_folder = path.join(self.virtual_gallery_folder, '1.0.0')

    def test_import(self):
        image_transfer_action = TransferAction.skip
        with tempfile.TemporaryDirectory() as tmpdirname:
            configuration = 'all'
            light_range = [1, 4]
            loop_range = [1]
            camera_range = list(range(0, 6))
            occlusion_range = [2, 3]
            as_rig = False

            import_virtual_gallery(self.virtual_gallery_1_0_0_folder, configuration,
                                   light_range, loop_range, camera_range, occlusion_range, as_rig,
                                   image_transfer_action,
                                   tmpdirname, force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.virtual_gallery_folder, 'kapture/all'))
            imported_virtual_gallery_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_virtual_gallery_data, expected_kdata))

        with tempfile.TemporaryDirectory() as tmpdirname:
            configuration = 'all'
            light_range = [4]
            loop_range = [1]
            camera_range = [2]
            occlusion_range = [3]
            as_rig = False

            import_virtual_gallery(self.virtual_gallery_1_0_0_folder, configuration,
                                   light_range, loop_range, camera_range, occlusion_range, as_rig,
                                   image_transfer_action,
                                   tmpdirname, force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.virtual_gallery_folder, 'kapture/reduced'))
            imported_virtual_gallery_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_virtual_gallery_data, expected_kdata))

        with tempfile.TemporaryDirectory() as tmpdirname:
            configuration = 'training'
            light_range = [1, 4]
            loop_range = [1]
            camera_range = list(range(0, 6))
            occlusion_range = [2, 3]
            as_rig = False

            import_virtual_gallery(self.virtual_gallery_1_0_0_folder, configuration,
                                   light_range, loop_range, camera_range, occlusion_range, as_rig,
                                   image_transfer_action,
                                   tmpdirname, force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.virtual_gallery_folder, 'kapture/training'))
            imported_virtual_gallery_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_virtual_gallery_data, expected_kdata))

        with tempfile.TemporaryDirectory() as tmpdirname:
            configuration = 'training'
            light_range = [1, 4]
            loop_range = [1]
            camera_range = list(range(0, 6))
            occlusion_range = [2, 3]
            as_rig = True

            import_virtual_gallery(self.virtual_gallery_1_0_0_folder, configuration,
                                   light_range, loop_range, camera_range, occlusion_range, as_rig,
                                   image_transfer_action,
                                   tmpdirname, force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.virtual_gallery_folder, 'kapture/training'))
            imported_virtual_gallery_data = kapture_from_dir(tmpdirname)
            self.assertFalse(equal_kapture(imported_virtual_gallery_data, expected_kdata))

            rigs_remove_inplace(imported_virtual_gallery_data.trajectories, imported_virtual_gallery_data.rigs)
            self.assertTrue(equal_kapture(imported_virtual_gallery_data, expected_kdata))

        with tempfile.TemporaryDirectory() as tmpdirname:
            configuration = 'testing'
            light_range = [1, 4]
            loop_range = [1]
            camera_range = list(range(0, 6))
            occlusion_range = [2, 3]
            as_rig = False

            import_virtual_gallery(self.virtual_gallery_1_0_0_folder, configuration,
                                   light_range, loop_range, camera_range, occlusion_range, as_rig,
                                   image_transfer_action,
                                   tmpdirname, force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.virtual_gallery_folder, 'kapture/testing'))
            imported_virtual_gallery_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_virtual_gallery_data, expected_kdata))


if __name__ == '__main__':
    unittest.main()



# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

"""
Import all types of UTBM sensors information files to make the sensors list.
These files are published on github at:
https://github.com/epan-utbm/utbm_robocar_dataset/tree/baselines/camera_info

The files have been produced by the ROS camera_calibration package:
https://wiki.ros.org/camera_calibration
"""

from typing import List, Union

from kapture.core.Sensors import Sensors
from kapture.utils.open_cv import import_opencv_camera_calibration

#  UTBM specific constants values

# Bumblebee2 cameras ROS topics and identifiers in same order:
TOPICS_BB2 = ['/BB2_9211442/left/image_raw',
              '/BB2_9211442/right/image_raw']
BB2_CAMERA_IDENTIFIERS = ['bb2_left', 'bb2_right']

# Bumblebee XB3 cameras ROS topics and identifiers in same order:
TOPICS_XB3 = ['/BBX3_7140017/camera/center/image_raw',
              '/BBX3_7140017/camera/left/image_raw',
              '/BBX3_7140017/camera/right/image_raw']
XB3_CAMERA_IDENTIFIERS = ['bb_xb3_center', 'bb_xb3_left', 'bb_xb3_right']


def import_utbm_sensors(camera_info_file_paths: Union[str, List[str]]) -> Sensors:
    """
    Import all camera info.

    :param camera_info_file_paths: the list of camera info file paths
    :return: kapture Sensors with all camera defined
    """
    sensors = Sensors()
    if isinstance(camera_info_file_paths, str):
        camera_info_file_paths = [camera_info_file_paths]
    for camera_info_file in camera_info_file_paths:
        sensor = import_opencv_camera_calibration(camera_info_file)
        sensors[sensor.name] = sensor
    return sensors



#!/usr/bin/env python3
# Copyright 2021-present NAVER Corp. Under BSD 3-clause license

import os
import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
from kapture.algo.compare import equal_kapture
from kapture.io.csv import kapture_from_dir
from kapture.io.records import TransferAction
from tools.kapture_import_image_list_with_poses import import_image_list_with_poses


class TestImageList(unittest.TestCase):

    def setUp(self):
        self._samples_folder = path.abspath(path.join(path.dirname(__file__),  '../samples/cour_carree_markers/'))
        self._images_list_with_poses_file = path.join(self._samples_folder, 'map/images_with_poses.txt')
        self._images_folder = path.join(self._samples_folder, 'map')
        self._camera_model_path = path.join(path.join(self._samples_folder, 'map'), 's10.yml')
        self._tempdir = tempfile.TemporaryDirectory()
        self._kapture_path = path.join(self._tempdir.name, 'kapture_local')
        os.makedirs(self._kapture_path, exist_ok=True)

    def tearDown(self) -> None:
        self._tempdir.cleanup()

    def test_import_images_missing_file(self):
        self.assertRaises(FileNotFoundError, import_image_list_with_poses,
                          '', self._images_folder, self._camera_model_path,
                          self._kapture_path, True, TransferAction.copy)

    def test_import_images_missing_dir(self):
        self.assertRaises(ValueError, import_image_list_with_poses,
                          self._images_list_with_poses_file, '', self._camera_model_path,
                          self._kapture_path, True, TransferAction.copy)

    def test_import_with_images(self):
        import_image_list_with_poses(self._images_list_with_poses_file, self._images_folder, self._camera_model_path,
                                     self._kapture_path, True, TransferAction.copy)

        ref_kapture_path = path.join(self._samples_folder, 'kapture_images')
        expected_kapture = kapture_from_dir(ref_kapture_path)
        imported_kapture = kapture_from_dir(self._kapture_path)
        self.assertTrue(equal_kapture(imported_kapture, expected_kapture))

    def test_import_images_no_camera(self):
        import_image_list_with_poses(self._images_list_with_poses_file, self._images_folder, '',
                                     self._kapture_path, True, TransferAction.copy)

        ref_kapture_path = path.join(self._samples_folder, 'kapture_images_no_camera')
        expected_kapture = kapture_from_dir(ref_kapture_path)
        imported_kapture = kapture_from_dir(self._kapture_path)
        self.assertTrue(equal_kapture(imported_kapture, expected_kapture))

    def test_import_no_images(self):
        import_image_list_with_poses(self._images_list_with_poses_file, '', self._camera_model_path,
                                     self._kapture_path, True, TransferAction.copy, True)

        ref_kapture_path = path.join(self._samples_folder, 'kapture_no_images')
        expected_kapture = kapture_from_dir(ref_kapture_path)
        imported_kapture = kapture_from_dir(self._kapture_path)
        self.assertTrue(equal_kapture(imported_kapture, expected_kapture))

    def test_import_no_images_nor_camera(self):
        import_image_list_with_poses(self._images_list_with_poses_file, '', '',
                                     self._kapture_path, True, TransferAction.copy, True)

        ref_kapture_path = path.join(self._samples_folder, 'kapture_no_images_nor_camera')
        expected_kapture = kapture_from_dir(ref_kapture_path)
        imported_kapture = kapture_from_dir(self._kapture_path)
        self.assertTrue(equal_kapture(imported_kapture, expected_kapture))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import os.path as path
import tempfile
import numpy as np
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.records import TransferAction, get_image_fullpath
from kapture.io.csv import kapture_from_dir
from kapture.io.features import image_keypoints_from_file, image_descriptors_from_file
from kapture.algo.compare import equal_poses
from kapture.utils.paths import path_secure
# tools
from kapture.converter.colmap.import_colmap import import_colmap, import_colmap_database, \
    import_colmap_from_reconstruction_files  # noqa: E402
from kapture.converter.colmap.export_colmap import export_colmap
from kapture.converter.colmap.export_colmap_rigs import export_colmap_rig_json
from kapture.converter.colmap.import_colmap_rigs import import_colmap_rig_json
from kapture.converter.colmap.cameras import get_camera_kapture_id_from_colmap_id
from kapture.converter.colmap.database import COLMAPDatabase
from kapture.converter.colmap.database_extra import exists_table

here_dirpath = path.abspath(path.dirname(__file__))


class TestColmapPrimitives(unittest.TestCase):
    def test_exists_table(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdirname:
            colmap_db_path = path.join(tmpdirname, 'colmap.db')
            colmap_db = COLMAPDatabase.connect(colmap_db_path)
            self.assertFalse(exists_table('two_view_geometries', colmap_db))
            colmap_db.create_tables()
            self.assertTrue(exists_table('two_view_geometries', colmap_db))
            colmap_db.close()


class TestColmapImportT265(unittest.TestCase):
    def setUp(self) -> None:
        self._sample_dirpath = path.join(here_dirpath, '..', 'samples', 't265', 'colmap')
        self._tempdir = tempfile.TemporaryDirectory()
        self._database_filepath = path.join(self._sample_dirpath, 'colmap.db')
        self._reconstruction_path = self._sample_dirpath
        self._kapture_dirpath = path.join(self._tempdir.name, 'kapture')

    def tearDown(self) -> None:
        self._tempdir.cleanup()

    def test_t265_db_only(self):
        kapture_data = import_colmap_database(self._database_filepath, self._kapture_dirpath,
                                              no_geometric_filtering=True)

        # check the numbers
        self.assertEqual(2, len(kapture_data.sensors))
        self.assertEqual(6, len(kapture_data.trajectories))
        self.assertEqual(6, len(kapture_data.records_camera))

        # check camera ids
        camera_ids_expected = set(['cam_00001', 'cam_00002'])  # may evolve in future, not crucial
        camera_ids_actual = set(kapture_data.sensors.keys())
        self.assertEqual(camera_ids_expected, camera_ids_actual)
        # check camera ids consistent in trajectories
        camera_ids_trajectories = set(cam_id for _, cam_id, _ in kapture.flatten(kapture_data.trajectories))
        self.assertEqual(camera_ids_actual, camera_ids_trajectories)
        # check camera ids consistent in records_camera
        camera_ids_records = set(cam_id for _, cam_id, _ in kapture.flatten(kapture_data.records_camera))
        self.assertEqual(camera_ids_actual, camera_ids_records)

        # check camera parameters
        cam1 = kapture_data.sensors['cam_00001']
        self.assertIsInstance(cam1, kapture.Camera)
        self.assertEqual(kapture.SensorType.camera.name, cam1.sensor_type)
        self.assertEqual(kapture.CameraType.OPENCV_FISHEYE, cam1.camera_type)
        params_expected = [848.0, 800.0, 284.468, 285.51, 424.355, 393.742, 0.0008, 0.031, -0.03, 0.005]
        self.assertAlmostEqual(params_expected, cam1.camera_params)

        # check records
        timestamp, cam_id, image = next(kapture.flatten(kapture_data.records_camera, is_sorted=True))
        self.assertEqual(1, timestamp)
        self.assertEqual('cam_00002', cam_id)
        self.assertEqual('rightraw/frame_000000001.jpg', image)

        # check trajectories
        timestamp, cam_id, pose = next(kapture.flatten(kapture_data.trajectories, is_sorted=True))
        self.assertEqual(1, timestamp)
        self.assertEqual('cam_00002', cam_id)
        pose_expected = kapture.PoseTransform(r=[0.9540331248716523, -0.03768128483784883, -0.2972570621910482,
                                                 -0.0062565444214723875],
                                              t=[2.7109402281860904, 0.13236653865769618, -2.868626176500939])
        self.assertTrue(equal_poses(pose_expected, pose))

        # this sample has no keypoints, descriptors nor matches
        self.assertFalse(path.exists(path.join(self._kapture_dirpath, 'reconstruction')))


class TestColmapMaupertuis(unittest.TestCase):
    def setUp(self) -> None:
        self._tempdir = tempfile.TemporaryDirectory()
        self._temp_dirpath = self._tempdir.name
        self._sample_dirpath = path.join(here_dirpath, '..', 'samples', 'maupertuis')
        self._colmap_dirpath = path.join(self._sample_dirpath, 'colmap')
        self._images_filepath = path.join(self._colmap_dirpath, 'images')
        self._database_filepath = path.join(self._colmap_dirpath, 'colmap.db')
        self._reconstruction_path = path.join(self._colmap_dirpath, 'reconstruction')
        self._kapture_dirpath = path.normpath(path.join(self._sample_dirpath, 'kapture'))
        self.keypoints_type = 'SIFT'
        self.descriptors_type = 'SIFT'

    def tearDown(self) -> None:
        self._tempdir.cleanup()

    def test_maupertuis_import_db_only(self):
        kapture_data = import_colmap_database(self._database_filepath, self._temp_dirpath,
                                              self.keypoints_type,
                                              self.descriptors_type,
                                              no_geometric_filtering=True)

        # check the numbers
        self.assertIsNone(kapture_data.trajectories)
        self.assertIsNone(kapture_data.points3d)
        self.assertIsNone(kapture_data.records_lidar)
        self.assertIsNone(kapture_data.records_wifi)
        self.assertIsNone(kapture_data.records_gnss)
        self.assertEqual(1, len(kapture_data.sensors))
        self.assertEqual(4, len(kapture_data.records_camera))
        self.assertEqual(4, len(kapture_data.keypoints[self.keypoints_type]))
        self.assertEqual(4, len(kapture_data.descriptors[self.descriptors_type]))
        self.assertEqual(6, len(kapture_data.matches[self.keypoints_type]))

        # check camera
        camera = kapture_data.sensors['cam_00001']
        self.assertEqual(kapture.SensorType.camera.name, camera.sensor_type)
        self.assertEqual(kapture.CameraType.SIMPLE_PINHOLE, camera.camera_type)
        self.assertAlmostEqual(camera.camera_params, [1919.0, 1079.0, 2302.7999999999997, 959.5, 539.5])

        # check snapshots
        snapshots = kapture_data.records_camera
        self.assertTrue(all('cam_00001' in ts for ts in snapshots.values()))
        self.assertEqual(['00.jpg', '01.jpg', '02.jpg', '03.jpg'],
                         [filename for _, _, filename in kapture.flatten(snapshots, True)])

        # check keypoints
        keypoints = kapture_data.keypoints[self.keypoints_type]
        self.assertEqual(np.float32, keypoints.dtype)
        self.assertEqual(6, keypoints.dsize)
        self.assertEqual({'00.jpg', '01.jpg', '02.jpg', '03.jpg'}, keypoints)
        keypoints_filepaths_actual = kapture.io.features.keypoints_to_filepaths(keypoints,
                                                                                self.keypoints_type,
                                                                                self._temp_dirpath)
        keypoints_filepaths_expected = {
            f'{i:02d}.jpg': path_secure(f'{self._temp_dirpath}/reconstruction/keypoints'
                                        f'/{self.keypoints_type}/{i:02d}.jpg.kpt')
            for i in [0, 1, 2, 3]}
        self.assertDictEqual(keypoints_filepaths_actual, keypoints_filepaths_expected)
        # check a keypoints file
        image_keypoints_filepaths = sorted(
            kapture.io.features.keypoints_to_filepaths(keypoints, self.keypoints_type, self._temp_dirpath).values())
        image_keypoints = image_keypoints_from_file(image_keypoints_filepaths[0], keypoints.dtype, keypoints.dsize)
        self.assertEqual((6424, 6), image_keypoints.shape)
        self.assertAlmostEqual([1290.908447265625, 4.156360626220703, -1.3475048542022705,
                                1.4732409715652466, -1.4732409715652466, -1.3475048542022705],
                               image_keypoints[0].tolist())

        self.assertAlmostEqual([1381.316650390625, 668.8056640625, 59.981021881103516,
                                46.423213958740234, -46.423213958740234, 59.981021881103516],
                               image_keypoints[-1].tolist())

        # check descriptors
        descriptors = kapture_data.descriptors[self.descriptors_type]
        self.assertEqual(np.uint8, descriptors.dtype)
        self.assertEqual(128, descriptors.dsize)
        self.assertEqual({'00.jpg', '01.jpg', '02.jpg', '03.jpg'}, descriptors)
        descriptors_filepaths_actual = kapture.io.features.descriptors_to_filepaths(descriptors,
                                                                                    self.descriptors_type,
                                                                                    self._temp_dirpath)
        descriptors_filepaths_expected = {
            f'{i:02d}.jpg': path_secure(f'{self._temp_dirpath}/reconstruction/descriptors/'
                                        f'{self.descriptors_type}/{i:02d}.jpg.desc')
            for i in [0, 1, 2, 3]}
        self.assertDictEqual(descriptors_filepaths_actual, descriptors_filepaths_expected)
        # check a descriptors file
        image_descriptors_filepaths = sorted(kapture.io.features.descriptors_to_filepaths(descriptors,
                                                                                          self.descriptors_type,
                                                                                          self._temp_dirpath).values())
        image_descriptors = image_descriptors_from_file(
            image_descriptors_filepaths[0], descriptors.dtype, descriptors.dsize)
        self.assertEqual(image_keypoints.shape[0], image_descriptors.shape[0])

        # check matches
        matches = kapture_data.matches[self.keypoints_type]
        self.assertEqual({('01.jpg', '03.jpg'), ('00.jpg', '02.jpg'),
                          ('00.jpg', '03.jpg'), ('02.jpg', '03.jpg'),
                          ('00.jpg', '01.jpg'), ('01.jpg', '02.jpg')},
                         set(matches))

    def test_maupertuis_import_txt_only(self):
        kapture_data = import_colmap_from_reconstruction_files(
            self._reconstruction_path, self._temp_dirpath, self.keypoints_type, skip=set())

        # check the numbers
        self.assertEqual(1, len(kapture_data.sensors))
        self.assertEqual(4, len(kapture_data.trajectories))
        self.assertEqual(4, len(kapture_data.records_camera))
        self.assertIs(kapture_data.records_lidar, None)
        self.assertIs(kapture_data.records_wifi, None)
        self.assertEqual(4, len(kapture_data.keypoints[self.keypoints_type]))
        self.assertIs(kapture_data.descriptors, None)
        self.assertIs(kapture_data.matches, None)
        self.assertEqual(1039, len(kapture_data.points3d))
        self.assertEqual(1039, len(kapture_data.observations))

        # check camera
        camera = kapture_data.sensors['cam_00001']
        self.assertEqual(kapture.SensorType.camera.name, camera.sensor_type)
        self.assertEqual(kapture.CameraType.SIMPLE_PINHOLE, camera.camera_type)
        self.assertAlmostEqual(camera.camera_params, [1919.0, 1079.0, 1847.53, 959.5, 539.5])

        # check snapshots
        snapshots = kapture_data.records_camera
        self.assertTrue(all('cam_00001' in ts for ts in snapshots.values()))
        self.assertEqual(['00.jpg', '01.jpg', '02.jpg', '03.jpg'],
                         [filename for _, _, filename in kapture.flatten(snapshots, True)])

        # check trajectories
        trajectory = kapture_data.trajectories
        self.assertTrue(all('cam_00001' in ts for ts in trajectory.values()))
        self.assertTrue(all(pose.r is not None and pose.t is not None
                            for ts in trajectory.values()
                            for pose in ts.values()))
        self.assertTrue(equal_poses(trajectory[1, 'cam_00001'],
                                    kapture.PoseTransform(r=[0.998245, -0.000889039, -0.0384732, -0.045019],
                                                          t=[3.24777, -2.58119, -0.0457181])))

        # check points3d
        self.assertEqual((1039, 6), kapture_data.points3d.shape)
        self.assertAlmostEqual([-2.39675, 4.62278, 13.2759, 57.0, 57.0, 49.0],
                               kapture_data.points3d[0].tolist())

        # check observations
        observations = kapture_data.observations
        # self.assertEqual(4, len(observations[0]))
        self.assertEqual({('01.jpg', 4561), ('02.jpg', 3389), ('00.jpg', 4975), ('03.jpg', 3472)},
                         set(observations[0]['SIFT']))

    def test_maupertuis_import(self):
        kapture_data = import_colmap(self._temp_dirpath, self._database_filepath,
                                     self._reconstruction_path, self._images_filepath,
                                     None, self.keypoints_type, self.descriptors_type,
                                     force_overwrite_existing=True,
                                     images_import_strategy=TransferAction.copy,
                                     no_geometric_filtering=True)

        # check the numbers
        self.assertEqual(1, len(kapture_data.sensors))
        self.assertEqual(4, len(kapture_data.trajectories))
        self.assertEqual(4, len(kapture_data.records_camera))
        self.assertIs(kapture_data.records_lidar, None)
        self.assertIs(kapture_data.records_wifi, None)
        self.assertIs(kapture_data.records_gnss, None)
        self.assertEqual(4, len(kapture_data.keypoints[self.keypoints_type]))
        self.assertEqual(4, len(kapture_data.descriptors[self.descriptors_type]))
        self.assertEqual(6, len(kapture_data.matches[self.keypoints_type]))
        self.assertEqual(1039, len(kapture_data.points3d))
        self.assertEqual(1039, len(kapture_data.observations))

        # compare against golden kapture
        kapture_data_golden = kapture_from_dir(self._kapture_dirpath)
        # drop GPS, Wifi, Lidar
        kapture_data.records_lidar = None
        kapture_data.records_wifi = None
        kapture_data_golden.records_gnss = None
        kapture_data_golden.sensors = kapture.Sensors({
            sensor_id: sensor
            for sensor_id, sensor in kapture_data_golden.sensors.items()
            if isinstance(sensor, kapture.Camera)
        })

        # compare
        equivalence = kapture.algo.compare.equal_kapture(kapture_data, kapture_data_golden)
        self.assertTrue(equivalence)
        # Check images copy
        all_records_cameras = list(kapture.flatten(kapture_data.records_camera))
        for _, _, name in all_records_cameras:
            image_path = get_image_fullpath(self._temp_dirpath, name)
            self.assertTrue(path.isfile(image_path), f"image link {image_path}")

    def test_maupertuis_export_db_only(self):
        # export/import and check
        colmap_db_filepath = path.join(self._temp_dirpath, 'colmap.db')
        export_colmap(self._kapture_dirpath, colmap_db_filepath, None,
                      self.keypoints_type, self.descriptors_type, None, True)

        kapture_data = import_colmap(self._temp_dirpath, colmap_db_filepath,
                                     keypoints_type=self.keypoints_type,
                                     descriptors_type=self.descriptors_type,
                                     force_overwrite_existing=True,
                                     no_geometric_filtering=True)

        # check the numbers
        self.assertEqual(1, len(kapture_data.sensors))
        self.assertEqual(4, len(kapture_data.trajectories))
        self.assertEqual(4, len(kapture_data.records_camera))
        self.assertIs(kapture_data.records_lidar, None)
        self.assertIs(kapture_data.records_wifi, None)
        self.assertEqual(4, len(kapture_data.keypoints[self.keypoints_type]))
        self.assertEqual(4, len(kapture_data.descriptors[self.descriptors_type]))
        self.assertEqual(6, len(kapture_data.matches[self.keypoints_type]))
        self.assertIs(kapture_data.points3d, None)
        self.assertIs(kapture_data.observations, None)

    def test_maupertuis_export(self):
        # export/import and check
        colmap_db_filepath = path.join(self._temp_dirpath, 'colmap.db')
        colmap_txt_filepath = path.join(self._temp_dirpath, 'dense')
        export_colmap(self._kapture_dirpath, colmap_db_filepath, colmap_txt_filepath,
                      self.keypoints_type,
                      self.descriptors_type,
                      None, True)

        kapture_data = import_colmap(self._temp_dirpath, colmap_db_filepath, colmap_txt_filepath,
                                     keypoints_type=self.keypoints_type,
                                     descriptors_type=self.descriptors_type,
                                     force_overwrite_existing=True,
                                     no_geometric_filtering=True)

        # check the numbers
        self.assertEqual(1, len(kapture_data.sensors))
        self.assertEqual(4, len(kapture_data.trajectories))
        self.assertEqual(4, len(kapture_data.records_camera))
        self.assertIs(kapture_data.records_lidar, None)
        self.assertIs(kapture_data.records_wifi, None)
        self.assertIs(kapture_data.records_gnss, None)
        self.assertEqual(4, len(kapture_data.keypoints[self.keypoints_type]))
        self.assertEqual(4, len(kapture_data.descriptors[self.descriptors_type]))
        self.assertEqual(6, len(kapture_data.matches[self.keypoints_type]))
        self.assertEqual(1039, len(kapture_data.points3d))
        self.assertEqual(1039, len(kapture_data.observations))

        # compare against golden kapture
        kapture_data_golden = kapture_from_dir(self._kapture_dirpath)
        # drop GPS, Wifi, Lidar
        kapture_data.records_lidar = None
        kapture_data.records_wifi = None
        kapture_data_golden.records_gnss = None
        kapture_data_golden.sensors = kapture.Sensors({
            sensor_id: sensor
            for sensor_id, sensor in kapture_data_golden.sensors.items()
            if isinstance(sensor, kapture.Camera)
        })

        # compare
        equivalence = kapture.algo.compare.equal_kapture(kapture_data, kapture_data_golden)
        self.assertTrue(equivalence)


class TestColmapRig(unittest.TestCase):
    def test_export(self):
        # the rig
        rigs = kapture.Rigs()
        rigs['rig0', get_camera_kapture_id_from_colmap_id(0)] = kapture.PoseTransform()
        rigs['rig0', get_camera_kapture_id_from_colmap_id(1)] = kapture.PoseTransform()
        # the records
        records_camera = kapture.RecordsCamera()
        records_camera[0, get_camera_kapture_id_from_colmap_id(0)] = 'camL/0000.jpg'
        records_camera[0, get_camera_kapture_id_from_colmap_id(1)] = 'camR/0000.jpg'
        records_camera[1, get_camera_kapture_id_from_colmap_id(0)] = 'camL/0001.jpg'
        records_camera[1, get_camera_kapture_id_from_colmap_id(1)] = 'camR/0001.jpg'
        # expect
        expected_rigs = [{
            "cameras": [
                {
                    "camera_id": 0,
                    "image_prefix": "camL"
                },
                {
                    "camera_id": 1,
                    "image_prefix": "camR"
                }
            ],
            "ref_camera_id": 0
        }]

        colmap_camera_ids = {get_camera_kapture_id_from_colmap_id(i): i for i in range(2)}
        colmap_rigs = export_colmap_rig_json(rigs, records_camera, colmap_camera_ids)
        self.assertEqual(colmap_rigs, expected_rigs)

    def test_import(self):
        colmap_rigs = [{
            "cameras": [
                {
                    "camera_id": 0,
                    "image_prefix": "camL"
                },
                {
                    "camera_id": 1,
                    "image_prefix": "camR"
                }
            ],
            "ref_camera_id": 0
        }]

        rigs_kapture, reconstructed_images, reconstructed_trajectories = import_colmap_rig_json(rigs_colmap=colmap_rigs)
        self.assertEqual([('rig0', get_camera_kapture_id_from_colmap_id(0)),
                          ('rig0', get_camera_kapture_id_from_colmap_id(1))],
                         rigs_kapture.key_pairs())
        self.assertIsNone(reconstructed_images)
        self.assertIsNone(reconstructed_trajectories)

        # the records
        images = kapture.RecordsCamera()
        images[0, get_camera_kapture_id_from_colmap_id(0)] = 'camL/0000.jpg'
        images[1, get_camera_kapture_id_from_colmap_id(1)] = 'camR/0000.jpg'
        images[2, get_camera_kapture_id_from_colmap_id(0)] = 'camL/0001.jpg'
        images[3, get_camera_kapture_id_from_colmap_id(1)] = 'camR/0001.jpg'
        rigs_kapture, reconstructed_images, reconstructed_trajectories = import_colmap_rig_json(
            rigs_colmap=colmap_rigs, images=images)
        # check timestamps has been recovered.
        self.assertEqual([(0, get_camera_kapture_id_from_colmap_id(0)),
                          (0, get_camera_kapture_id_from_colmap_id(1)),
                          (1, get_camera_kapture_id_from_colmap_id(0)),
                          (1, get_camera_kapture_id_from_colmap_id(1))],
                         reconstructed_images.key_pairs())

        # trajectories
        trajectories = kapture.Trajectories()
        trajectories[0, get_camera_kapture_id_from_colmap_id(0)] = kapture.PoseTransform()
        trajectories[1, get_camera_kapture_id_from_colmap_id(1)] = kapture.PoseTransform()
        trajectories[2, get_camera_kapture_id_from_colmap_id(0)] = kapture.PoseTransform()
        trajectories[3, get_camera_kapture_id_from_colmap_id(1)] = kapture.PoseTransform()
        rigs_kapture, reconstructed_images, reconstructed_trajectories = import_colmap_rig_json(
            rigs_colmap=colmap_rigs, images=images, trajectories=trajectories)
        self.assertEqual([(0, get_camera_kapture_id_from_colmap_id(0)),
                          (0, get_camera_kapture_id_from_colmap_id(1)),
                          (1, get_camera_kapture_id_from_colmap_id(0)),
                          (1, get_camera_kapture_id_from_colmap_id(1))],
                         reconstructed_trajectories.key_pairs())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import shutil
import tempfile
import unittest
import sys
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.csv import kapture_from_dir
from kapture.io.records import TransferAction
from tools.kapture_import_image_folder import import_image_folder


class TestImageFolder(unittest.TestCase):
    def setUp(self):
        samples_folder = path.abspath(path.join(path.dirname(__file__),  '../samples/Aachen-Day-Night'))
        self.images_folder = path.join(samples_folder, 'images_upright', 'query', 'day')

    @unittest.skipIf(sys.platform.startswith("win"), "Do not work on Windows without admin rights")
    def test_import_image_folder(self):
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_image_folder(
                images_path=self.images_folder,
                kapture_path=tmpdirname,
                force_overwrite_existing=True,
                images_import_method=TransferAction.link_absolute)

            kapture_result = kapture_from_dir(tmpdirname)

        # sensors
        sensors = list(kapture.flatten(kapture_result.sensors, is_sorted=True))
        self.assertEqual(4, len(sensors))
        first_sensor = sensors[0][1]
        self.assertTrue(isinstance(first_sensor, kapture.Sensor))
        self.assertTrue(isinstance(first_sensor, kapture.Camera))
        self.assertEqual(kapture.CameraType.UNKNOWN_CAMERA, first_sensor.camera_type)
        # images
        images = list(kapture.flatten(kapture_result.records_camera, is_sorted=True))
        self.assertEqual(4, len(images))
        first_image = images[0]
        self.assertEqual((0, 'sensor0', 'milestone/2011-12-17_14-28-15_257.jpg'), first_image)

    @unittest.skipIf(sys.platform.startswith("win"), "Do not work on Windows without admin rights")
    def test_import_image_folder_with_single_sensor(self):
        # creates a folder with both images and sensors.txt
        with tempfile.TemporaryDirectory() as tmpdirname_in, tempfile.TemporaryDirectory() as tmpdirname_out:
            # copy images
            shutil.copytree(self.images_folder, tmpdirname_in, dirs_exist_ok=True)
            # copy sensors.txt
            sensors_file_path = path.join(tmpdirname_in, 'sensors.txt')
            with open(sensors_file_path, 'wt') as f:
                f.write("""
                cam0, , camera, RADIAL, 2832, 2128, 2986.7801886485645, 1463.2779641275515, 1112.2674656244137, \
                -0.2465279459531899, 0.2675790139425493
                """)

            import_image_folder(tmpdirname_in, tmpdirname_out, True, TransferAction.link_absolute)

            kapture_result = kapture_from_dir(tmpdirname_out)

        sensors = list(kapture.flatten(kapture_result.sensors, is_sorted=True))
        # sensors
        self.assertEqual(1, len(sensors))
        camera_id = sensors[0][0]
        camera = sensors[0][1]
        self.assertEqual('cam0', camera_id)
        self.assertTrue(isinstance(camera, kapture.Sensor))
        self.assertTrue(isinstance(camera, kapture.Camera))
        self.assertEqual(kapture.CameraType.RADIAL, camera.camera_type)
        # images
        images = list(kapture.flatten(kapture_result.records_camera, is_sorted=True))
        self.assertEqual(4, len(images))
        first_image = images[0]
        self.assertEqual((0, 'cam0', 'milestone/2011-12-17_14-28-15_257.jpg'), first_image)

    @unittest.skipIf(sys.platform.startswith("win"), "Do not work on Windows without admin rights")
    def test_import_image_folder_with_multiple_sensors(self):
        # creates a folder with both images and sensors.txt
        with tempfile.TemporaryDirectory() as tmpdirname_in, tempfile.TemporaryDirectory() as tmpdirname_out:
            # copy images
            shutil.copytree(self.images_folder, tmpdirname_in, dirs_exist_ok=True)
            # make up sensors.txt
            sensors_file_path = path.join(tmpdirname_in, 'sensors.txt')
            with open(sensors_file_path, 'wt') as f:
                f.write("""
                milestone, , camera, RADIAL, 2832, 2128, 2986.7801886485645, 1463.2779641275515, 1112.2674656244137, \
                -0.2465279459531899, 0.2675790139425493
                nexus4, , camera, RADIAL, 2832, 2128, 2986.7801886485645, 1463.2779641275515, 1112.2674656244137, \
                -0.2465279459531899, 0.2675790139425493
                """)

            import_image_folder(tmpdirname_in, tmpdirname_out, True, TransferAction.link_absolute)

            kapture_result = kapture_from_dir(tmpdirname_out)

        sensors = list(kapture.flatten(kapture_result.sensors, is_sorted=True))
        # sensors
        self.assertEqual(2, len(sensors))
        camera_ids = [s[0] for s in sensors]
        cameras = [s[1] for s in sensors]
        self.assertEqual(['milestone', 'nexus4'], camera_ids)
        self.assertTrue(kapture.Sensor, cameras[0])
        self.assertTrue(kapture.Sensor, cameras[1])
        self.assertTrue(kapture.Camera, cameras[0])
        self.assertTrue(kapture.Camera, cameras[1])
        self.assertEqual(kapture.CameraType.RADIAL, cameras[0].camera_type)
        self.assertEqual(kapture.CameraType.RADIAL, cameras[1].camera_type)
        # images
        images = list(kapture.flatten(kapture_result.records_camera, is_sorted=True))
        self.assertEqual(4, len(images))
        first_image = images[0]
        self.assertEqual((0, 'milestone', 'milestone/2011-12-17_14-28-15_257.jpg'), first_image)



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

"""
Tests of the kapture core module.
"""

import unittest
import numpy as np
import quaternion
from copy import deepcopy
from datetime import datetime
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.algo.compare import equal_trajectories, equal_rigs, equal_poses


# FLATTEN ##############################################################################################################
class TestFlatten(unittest.TestCase):
    def test_non_dict(self):
        test_values = 5
        expected_list = [(test_values,)]
        actual_list = list(kapture.flatten(test_values, is_sorted=False))
        actual_list_sorted = list(kapture.flatten(test_values, is_sorted=True))
        self.assertEqual(actual_list, expected_list)
        self.assertEqual(actual_list_sorted, expected_list)

        test_values = [5.0, 7.3, 6.02]
        expected_list = [(5.0,), (7.3,), (6.02,)]
        expected_list_sorted = [(5.0,), (6.02,), (7.3,)]
        actual_list = list(kapture.flatten(test_values, is_sorted=False))
        actual_list_sorted = list(kapture.flatten(test_values, is_sorted=True))
        self.assertEqual(actual_list, expected_list)
        self.assertEqual(actual_list_sorted, expected_list_sorted)

        test_values = kapture.RecordWifiSignal(frequency=2500, rssi=-1.0)
        expected_list = [(test_values,)]
        actual_list = list(kapture.flatten(test_values, is_sorted=False))
        actual_list_sorted = list(kapture.flatten(test_values, is_sorted=True))
        self.assertEqual(actual_list, expected_list)
        self.assertEqual(actual_list_sorted, expected_list)

    def test_simple_dict(self):
        test_values = {'a': 1, 'c': 3, 'b': 4, 'z': 0, 'y': 9, 'd': 5}
        expected_list = [('a', 1), ('c', 3), ('b', 4), ('z', 0), ('y', 9), ('d', 5)]
        expected_list_sorted = [('a', 1), ('b', 4), ('c', 3), ('d', 5), ('y', 9), ('z', 0)]
        actual_list = list(kapture.flatten(test_values, is_sorted=False))
        actual_list_sorted = list(kapture.flatten(test_values, is_sorted=True))
        self.assertEqual(actual_list, expected_list)
        self.assertEqual(actual_list_sorted, expected_list_sorted)

    def test_dict_of_list(self):
        test_values = {'a': [1, 4, 3], 'z': [1, 0], 'y': [9, 5]}
        expected_list = [('a', 1), ('a', 4), ('a', 3), ('z', 1), ('z', 0), ('y', 9), ('y', 5)]
        expected_list_sorted = [('a', 1), ('a', 3), ('a', 4), ('y', 5), ('y', 9), ('z', 0), ('z', 1)]
        actual_list = list(kapture.flatten(test_values, is_sorted=False))
        actual_list_sorted = list(kapture.flatten(test_values, is_sorted=True))
        self.assertEqual(actual_list, expected_list)
        self.assertEqual(actual_list_sorted, expected_list_sorted)

    def test_nested_dict(self):
        test_values = {'a': {'a': 1, 'c': 3, 'b': 4}, 'c': {'z': 0, 'y': 9, 'd': 5}, 'b': {'u': 7, 'a': 13, 'c': 1}}
        expected_list = [('a', 'a', 1), ('a', 'c', 3), ('a', 'b', 4),
                         ('c', 'z', 0), ('c', 'y', 9), ('c', 'd', 5),
                         ('b', 'u', 7), ('b', 'a', 13), ('b', 'c', 1)]
        expected_list_sorted = [('a', 'a', 1), ('a', 'b', 4), ('a', 'c', 3),
                                ('b', 'a', 13), ('b', 'c', 1), ('b', 'u', 7),
                                ('c', 'd', 5), ('c', 'y', 9), ('c', 'z', 0)]
        actual_list = list(kapture.flatten(test_values, is_sorted=False))
        actual_list_sorted = list(kapture.flatten(test_values, is_sorted=True))
        self.assertEqual(actual_list, expected_list)
        self.assertEqual(actual_list_sorted, expected_list_sorted)


# POSES ################################################################################################################
class TestPoseTransformConstruct(unittest.TestCase):

    def test_canonicals_list(self):
        expect_r = [1., 0., 0., 0.]
        expect_t = [0., 0., 0.]
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r
        actual_t = pose.t
        # check r
        self.assertIsInstance(actual_r, np.quaternion)
        self.assertListEqual(quaternion.as_float_array(
            actual_r).tolist(), expect_r)
        # check t
        self.assertIsInstance(actual_t, np.ndarray)
        self.assertTupleEqual(actual_t.shape, (3, 1))
        self.assertListEqual(actual_t.flatten().tolist(), expect_t)

    def test_canonicals_numpy(self):
        expect_r = np.array([1., 0., 0., 0.])
        expect_t = np.array([0., 0., 0.])
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r
        actual_t = pose.t
        # check r
        self.assertIsInstance(actual_r, np.quaternion)
        self.assertListEqual(quaternion.as_float_array(
            actual_r).tolist(), expect_r.tolist())
        # check t
        self.assertIsInstance(actual_t, np.ndarray)
        self.assertTupleEqual(actual_t.shape, (3, 1))
        self.assertListEqual(actual_t.flatten().tolist(), expect_t.tolist())

    def test_canonicals_quaternion(self):
        expect_r = quaternion.from_float_array([1., 0., 0., 0.])
        expect_t = np.array([0., 0., 0.])
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r
        actual_t = pose.t
        # check r
        self.assertIsInstance(actual_r, np.quaternion)
        self.assertListEqual(quaternion.as_float_array(
            actual_r).tolist(), quaternion.as_float_array(expect_r).tolist())
        # check t
        self.assertIsInstance(actual_t, np.ndarray)
        self.assertTupleEqual(actual_t.shape, (3, 1))
        self.assertListEqual(actual_t.flatten().tolist(), expect_t.tolist())

    def test_none_list(self):
        expect_r = [None] * 4
        expect_t = [None] * 3
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r
        actual_t = pose.t
        # check r
        self.assertEqual(actual_r, None)
        # check t
        self.assertEqual(actual_t, None)

    def test_none(self):
        expect_r = None
        expect_t = None
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r
        actual_t = pose.t
        # check r
        self.assertEqual(actual_r, None)
        # check t
        self.assertEqual(actual_t, None)


class TestPoseTransformRaw(unittest.TestCase):

    def test_canonicals_list(self):
        expect_r = [1., 0., 0., 0.]
        expect_t = [0., 0., 0.]
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r_raw
        actual_t = pose.t_raw
        self.assertListEqual(actual_r, expect_r)
        self.assertListEqual(actual_t, expect_t)

    def test_none(self):
        expect_r = None
        expect_t = None
        pose = kapture.PoseTransform(expect_r, expect_t)
        actual_r = pose.r_raw
        actual_t = pose.t_raw
        self.assertEqual(actual_r, None)
        self.assertEqual(actual_t, None)


class TestPoseTransformApply(unittest.TestCase):
    def setUp(self):
        self.expected_points3d = np.arange(3 * 10).reshape(-1, 3)

    def test_identity(self):
        # default construct
        pose_identity = kapture.PoseTransform()
        actual_points3d = pose_identity.transform_points(
            self.expected_points3d)
        self.assertTrue(np.all(actual_points3d == self.expected_points3d))
        # explicit identity
        pose_identity = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        actual_points3d = pose_identity.transform_points(
            self.expected_points3d)
        self.assertTrue(np.all(actual_points3d == self.expected_points3d))

    def test_translation(self):
        pose_translate = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 10])
        actual_points3d = pose_translate.transform_points(
            self.expected_points3d)
        diff = (actual_points3d - self.expected_points3d)
        self.assertTrue(np.all(diff[:, 2] == 10))
        self.assertTrue(np.all(diff[:, 0:2] == 0))

    def test_rotation(self):
        r = quaternion.from_rotation_vector(
            [0, 0, np.pi])  # rotate 180° around Z axis
        # print(quaternion.as_rotation_matrix(r))
        pose_rotate = kapture.PoseTransform(r=r, t=[0, 0, 0])
        actual_points3d = pose_rotate.transform_points(self.expected_points3d)
        self.assertTrue(np.all(np.isclose(
            actual_points3d[:, 0:2], - self.expected_points3d[:, 0:2])))  # X, Y opposed
        self.assertTrue(np.all(np.isclose(
            actual_points3d[:, 2], self.expected_points3d[:, 2])))  # Z untouched


class TestPoseEquality(unittest.TestCase):
    def test_equal(self):
        pose_1 = kapture.PoseTransform(r=[1, 10, 0, 0], t=[0, 0, 10])
        pose_2 = kapture.PoseTransform(r=[1, 10, 0, 0], t=[0, 0, 10])
        self.assertEqual(pose_1, pose_2)
        self.assertNotEqual(pose_1, kapture.PoseTransform(t=None))
        self.assertNotEqual(pose_1, kapture.PoseTransform(r=None))
        pose_2 = kapture.PoseTransform(r=[1, 10, 0, 0], t=[0, 0.000009, 10])
        self.assertEqual(pose_1, pose_2)
        pose_2 = kapture.PoseTransform(r=[1, 10, 0, 0], t=[0, 0.22, 10])
        self.assertNotEqual(pose_1, pose_2)
        pose_2 = kapture.PoseTransform(r=[1, 10, 0, 0.011], t=[0, 0.0, 10])
        self.assertNotEqual(pose_1, pose_2)
        pose_2 = kapture.PoseTransform(r=[1, 10, 0, 0.01], t=[0, 0.0, 10])
        self.assertEqual(pose_1, pose_2)


# SENSOR ###############################################################################################################
class TestSensor(unittest.TestCase):
    def test_init(self):
        # test bare minimum
        sensor = kapture.Sensor('unknown', [])
        self.assertEqual(sensor.name, None)
        self.assertEqual(sensor.sensor_type, 'unknown')
        self.assertListEqual(sensor.sensor_params, [])

        # test typical camera
        sensor_name = 'GOPRO_FUSION'
        sensor_type = kapture.SensorType.camera.name
        #                 SIMPLE_PINHOLE,   w,   h,   f,  cx,  cy
        sensor_params = ['SIMPLE_PINHOLE', 640, 480, 100, 320, 240]
        sensor = kapture.Sensor(sensor_type, sensor_params, sensor_name)
        self.assertEqual(sensor.name, sensor_name)
        self.assertEqual(sensor.sensor_type, sensor_type)
        self.assertListEqual(sensor.sensor_params, [i for i in sensor_params])
        self.assertIsInstance(sensor.__repr__(), str)

        sensor = kapture.Camera(sensor_params[0], sensor_params[1:], sensor_name)
        self.assertEqual(sensor.name, sensor_name)
        self.assertEqual(sensor.sensor_type, sensor_type)
        self.assertEqual(sensor.camera_type, kapture.CameraType.SIMPLE_PINHOLE)
        self.assertListEqual(sensor.sensor_params, [str(i) for i in sensor_params])
        self.assertListEqual(sensor.camera_params, [float(i) for i in sensor_params[1:]])
        self.assertIsInstance(sensor.__repr__(), str)

    def test_update(self):
        pass
        # TODO
        # sensor = kapture.Sensor('unknown', [])
        # sensor.name = 'toto'


class TestCameraModel(unittest.TestCase):
    def test_model_params_fully_set(self):
        for sensor_model in kapture.CameraType:
            self.assertIn(sensor_model, kapture.CAMERA_TYPE_PARAMS_COUNT)
            self.assertIn(sensor_model.value,
                          kapture.CAMERA_TYPE_PARAMS_COUNT_FROM_NAME)

    def test_dict(self):
        model = kapture.CameraType.FULL_OPENCV
        self.assertEqual(model.value, "FULL_OPENCV")
        self.assertEqual(
            kapture.CAMERA_TYPE_PARAMS_COUNT[model], kapture.CAMERA_TYPE_PARAMS_COUNT_FROM_NAME[model.value])
        self.assertEqual(kapture.CAMERA_TYPE_PARAMS_COUNT[model], 14)


# SENSORS ##############################################################################################################
class TestSensors(unittest.TestCase):
    def test_init(self):
        sensors = kapture.Sensors()
        self.assertEqual(0, len(sensors))
        sensors['cam0'] = kapture.Sensor('unknown', [])
        self.assertEqual(1, len(sensors))
        self.assertIn('cam0', sensors)
        self.assertIn('unknown', sensors['cam0'].sensor_type)

    def test_type_checking(self):
        sensors = kapture.Sensors()
        invalid_sensor_id = tuple('a', )
        valid_sensor_id = 'cam0'
        invalid_sensor = int(0)
        valid_sensor = kapture.Sensor(kapture.SensorType.camera.name)
        self.assertRaises(TypeError, sensors.__setitem__, valid_sensor_id, invalid_sensor)
        self.assertRaises(TypeError, sensors.__setitem__, invalid_sensor_id, valid_sensor)
        self.assertRaises(TypeError, sensors.__setitem__, invalid_sensor_id, invalid_sensor)


# RIGS #################################################################################################################
class TestRigs(unittest.TestCase):
    def test_init(self):
        rigs = kapture.Rigs()
        rigs['rig0', 'cam0'] = kapture.PoseTransform()
        rigs['rig0', 'cam1'] = kapture.PoseTransform()
        rigs['rig1'] = {
            'cam2': kapture.PoseTransform(),
            'cam3': kapture.PoseTransform()
        }
        self.assertEqual(2, len(rigs))
        self.assertIn('rig0', rigs)
        self.assertIn('rig1', rigs)
        self.assertIn('cam0', rigs['rig0'])
        self.assertNotIn('cam0', rigs['rig1'])

    def test_type_checking(self):
        rigs = kapture.Rigs()
        valid_id, valid_pose = 'rig0', kapture.PoseTransform()
        invalid_id, invalid_pose = float(0), 'rig'

        self.assertRaises(TypeError, rigs.__setitem__, valid_id, valid_pose)
        self.assertRaises(TypeError, rigs.__setitem__, (valid_id, invalid_id), valid_pose)
        self.assertRaises(TypeError, rigs.__setitem__, (invalid_id, valid_id), valid_pose)
        self.assertRaises(TypeError, rigs.__setitem__, (valid_id, valid_id), invalid_pose)


# TRAJECTORIES #########################################################################################################
class TestTrajectories(unittest.TestCase):
    def test_init(self):
        timestamp1 = 0
        timestamp2 = 1
        device_id = 'cam0'
        pose = kapture.PoseTransform()
        traj = kapture.Trajectories()
        # pair assignment
        traj[(timestamp1, device_id)] = pose
        # dict assignment
        traj[timestamp2] = {device_id: pose}
        self.assertEqual(2, len(traj))
        self.assertEqual(traj[timestamp1],
                         traj[timestamp2])
        self.assertEqual(traj[(timestamp1, device_id)],
                         traj[(timestamp2, device_id)])

        # test __contains__
        self.assertIn(timestamp1, traj)
        self.assertIn((timestamp1, device_id), traj)
        self.assertIn(timestamp2, traj)
        self.assertIn((timestamp1, device_id), traj)

        self.assertNotIn((timestamp1, 'cam1'), traj)
        self.assertNotIn((2, device_id), traj)

    def test_type_checking(self):
        traj = kapture.Trajectories()
        valid_ts, valid_id, valid_pose = 0, 'cam0', kapture.PoseTransform()
        invalid_ts, invalid_id, invalid_pose = '0', float(0), 'pose'
        self.assertRaises(TypeError, traj.__setitem__, (invalid_ts, valid_id), valid_pose)
        self.assertRaises(TypeError, traj.__setitem__, (valid_ts, invalid_id), valid_pose)
        self.assertRaises(TypeError, traj.__setitem__, (valid_ts, valid_id), invalid_pose)
        self.assertRaises(TypeError, traj.__setitem__, (invalid_ts, invalid_id), invalid_pose)

        self.assertRaises(TypeError, traj.__setitem__, invalid_ts, {valid_id: valid_pose})
        self.assertRaises(TypeError, traj.__setitem__, valid_ts, {invalid_id: valid_pose})
        self.assertRaises(TypeError, traj.__setitem__, valid_ts, {valid_id: invalid_pose})
        self.assertRaises(TypeError, traj.__setitem__, invalid_ts, valid_pose)

        self.assertRaises(TypeError, traj.__contains__, invalid_ts, valid_id)
        self.assertRaises(TypeError, traj.__contains__, valid_ts, invalid_id)
        self.assertRaises(TypeError, traj.__contains__, invalid_ts, invalid_id)

        self.assertRaises(TypeError, traj.__delitem__, invalid_ts)
        self.assertRaises(TypeError, traj.__delitem__, (valid_ts, invalid_id))

    def test_inverse(self):
        pose1 = kapture.PoseTransform()
        pose2 = kapture.PoseTransform(
            r=[-0.09465784241766664, 0.4240600207396319, -0.6602839272564094, 0.6125664478512957],
            t=[-81525.97101792274, 6362571.7626688825, 216599.32985438255]
        )
        pose3 = kapture.PoseTransform(
            r=[-0.07509999120329255, 0.428687830189451, -0.6630694170354837, 0.6090368344187961],
            t=[49418.858719801996, 6364643.125801517, 157364.12971148116]
        )
        traj = kapture.Trajectories()
        traj[(0, 'dummy1')] = pose1
        traj[(1, 'dummy1')] = pose2
        traj[(0, 'dummy2')] = pose2
        traj[(1, 'dummy2')] = pose3
        traj_inv = traj.inverse()

        self.assertEqual(traj.key_pairs(), traj_inv.key_pairs())
        self.assertTrue(equal_poses(traj_inv[0, 'dummy1'], pose1))  # identity
        self.assertTrue(equal_poses(traj_inv[1, 'dummy1'], pose2.inverse()))  #
        self.assertTrue(equal_poses(traj_inv[1, 'dummy2'], pose3.inverse()))  #

    def test_rig_remove(self):
        rigs = kapture.Rigs()
        rigs['rig0', 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[100, 0, 0])
        rigs['rig0', 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[-100, 0, 0])
        trajectories = kapture.Trajectories()
        trajectories[0, 'rig0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        trajectories[1, 'rig0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 10])
        trajectories[2, 'rig0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 20])
        trajectories_ = kapture.rigs_remove(trajectories, rigs)
        # timestamps should be unchanged
        self.assertEqual(trajectories_.keys(), trajectories.keys())
        self.assertNotEqual(trajectories_.key_pairs(), trajectories.key_pairs())
        self.assertEqual(len(trajectories_.key_pairs()), len(trajectories.key_pairs()) * len(rigs.key_pairs()))
        self.assertIn((0, 'cam0'), trajectories_.key_pairs())
        self.assertIn((0, 'cam1'), trajectories_.key_pairs())
        self.assertIn((2, 'cam0'), trajectories_.key_pairs())
        self.assertIn((2, 'cam1'), trajectories_.key_pairs())
        self.assertAlmostEqual(trajectories_[2, 'cam1'].t_raw, [-100.0, 0.0, 20.0])
        self.assertAlmostEqual(trajectories_[2, 'cam1'].r_raw, [1.0, 0.0, 0.0, 0.0])

    def test_remove(self):
        trajectories = kapture.Trajectories()
        trajectories[0, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        trajectories[1, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 10])
        trajectories[2, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 20])
        trajectories[2, 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[10, 0, 20])
        self.assertEqual(len(trajectories), 3)
        self.assertEqual(len(trajectories[2]), 2)

        del trajectories[2, 'cam0']
        self.assertEqual(len(trajectories), 3)
        self.assertEqual(len(trajectories[2]), 1)

        del trajectories[1]
        self.assertEqual(len(trajectories), 2)
        del trajectories[2, 'cam1']
        self.assertEqual(len(trajectories), 1)

    def test_timestamps_list(self):
        trajectories = kapture.Trajectories()
        trajectories[2, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 20])
        trajectories[0, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        trajectories[2, 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[10, 0, 20])
        self.assertEqual(trajectories.timestamps_sorted_list(), [0, 2])
        trajectories[1, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 10])
        self.assertEqual(len(trajectories), 3)
        self.assertEqual(trajectories.timestamps_sorted_list(), [0, 1, 2])

        del trajectories[2, 'cam0']
        self.assertEqual(trajectories.timestamps_sorted_list(), [0, 1, 2])

        del trajectories[1]
        self.assertEqual(trajectories.timestamps_sorted_list(), [0, 2])
        del trajectories[2, 'cam1']
        self.assertEqual(trajectories.timestamps_sorted_list(), [0, ])

    def test_timestamps_length(self):
        trajectories = kapture.Trajectories()
        self.assertEqual(trajectories.timestamp_length(), -1)
        trajectories[1614362592378, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 20])
        trajectories[1614362592634, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        trajectories[1614362592378, 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[10, 0, 20])
        trajectories[1614362593123, 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[10, 0, 30])
        self.assertEqual(trajectories.timestamp_length(), 13)
        # Check that if we have timestamps of different precision, we can not compute a common length
        trajectories[1614362594, 'lidar0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        self.assertEqual(trajectories.timestamp_length(), -1)

    def test_pose_interpolation(self):
        trajectories = kapture.Trajectories()
        trajectories[1614362592000, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 0])
        trajectories[1614362592000, 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 10, 0])
        trajectories[1614362592500, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 10])
        trajectories[1614362593000, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 20])
        trajectories[1614362593000, 'cam1'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 10, 20])
        trajectories[1614362593500, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 30])
        trajectories[1614362594000, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 40])
        trajectories[1614362594500, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 50])
        trajectories[1614362595000, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 60])
        trajectories[1614362595500, 'cam0'] = kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 70])
        self.assertEqual(trajectories.timestamp_length(), 13)
        pose = trajectories.intermediate_pose(1614362592500, 'cam2', 1000000)
        self.assertIsNone(pose, "unknown device")
        pose = trajectories.intermediate_pose(1614362593000, 'cam0', 1000000)
        self.assertEqual(pose, kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 20]), "existing pose")
        pose = trajectories.intermediate_pose(1614362500000, 'cam0', 1000000)
        self.assertIsNone(pose, "time too far in past")
        pose = trajectories.intermediate_pose(1614362600000, 'cam0', 1000000)
        self.assertIsNone(pose, "time too far in future")
        pose = trajectories.intermediate_pose(1614362595250, 'cam0', 1000000)
        self.assertEqual(pose, kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 0, 65]))
        pose = trajectories.intermediate_pose(1614362592500, 'cam1', 1000000)
        self.assertEqual(pose, kapture.PoseTransform(r=[1, 0, 0, 0], t=[0, 10, 10]))
        pose = trajectories.intermediate_pose(1614362595250, 'cam1', 1000000)
        self.assertIsNone(pose, "not enough pose for cam1")


# REMOVE/RESTORE RIGS in TRAJECTORIES ##################################################################################
class TestTrajectoriesRig(unittest.TestCase):
    def setUp(self):
        self._rigs = kapture.Rigs()
        looking_not_straight = quaternion.from_rotation_vector([0, np.deg2rad(5.), 0])
        self._rigs['rig', 'phone'] = kapture.PoseTransform()  # do a nested rig
        self._rigs['phone', 'cam1'] = kapture.PoseTransform(t=[-10, 0, 0], r=looking_not_straight).inverse()
        self._rigs['phone', 'cam2'] = kapture.PoseTransform(t=[+10, 0, 0], r=looking_not_straight.inverse()).inverse()
        self._trajectories_rigs = kapture.Trajectories()
        for timestamp, ratio in enumerate(np.linspace(0., 1., num=8)):
            looking_around = quaternion.from_rotation_vector([0, np.deg2rad(360. * ratio), 0])
            self._trajectories_rigs[timestamp, 'rig'] = kapture.PoseTransform(t=[0, 0, -100.], r=looking_around)

        self._trajectories_cams = kapture.Trajectories()
        for timestamp, rig_id, pose_rig_from_world in kapture.flatten(self._trajectories_rigs, is_sorted=True):
            for rig_id2, cam_id, pose_cam_from_rig in kapture.flatten(self._rigs):
                if cam_id == 'phone':
                    continue
                pose_cam_from_world = kapture.PoseTransform.compose([pose_cam_from_rig, pose_rig_from_world])
                self._trajectories_cams[timestamp, cam_id] = pose_cam_from_world

    def test_rig_remove(self):
        trajectories_wo_rigs = kapture.rigs_remove(self._trajectories_rigs, self._rigs)
        self.assertTrue(equal_trajectories(trajectories_wo_rigs, self._trajectories_cams))

    def test_rig_remove_inplace(self):
        trajectories = deepcopy(self._trajectories_rigs)
        rigs = deepcopy(self._rigs)
        kapture.rigs_remove_inplace(trajectories, rigs)
        self.assertTrue(equal_trajectories(trajectories, self._trajectories_cams))
        self.assertTrue(equal_rigs(rigs, self._rigs))

    def test_rig_remove_inplace_consistency(self):
        # compare inplace and not inplace
        trajectories_inplace = deepcopy(self._trajectories_rigs)
        rigs_inplace = deepcopy(self._rigs)
        kapture.rigs_remove_inplace(trajectories_inplace, rigs_inplace)
        trajectories_not_inplace = kapture.rigs_remove(self._trajectories_rigs, self._rigs)
        self.assertTrue(equal_trajectories(trajectories_inplace, trajectories_not_inplace))
        # make sure rigs are untouched.
        self.assertTrue(equal_rigs(rigs_inplace, self._rigs))

    def test_rig_recover(self):
        trajectories_w_rigs = kapture.rigs_recover(self._trajectories_cams, self._rigs)
        self.assertTrue(equal_trajectories(trajectories_w_rigs, self._trajectories_rigs))

    def test_rig_recover_inplace(self):
        trajectories = deepcopy(self._trajectories_cams)
        rigs = deepcopy(self._rigs)
        kapture.rigs_recover_inplace(trajectories, rigs)
        self.assertTrue(equal_trajectories(trajectories, self._trajectories_rigs))
        self.assertTrue(equal_rigs(rigs, self._rigs))

    def test_rig_recover_inplace_consistency(self):
        # compare inplace and not inplace
        trajectories_inplace = deepcopy(self._trajectories_cams)
        rigs_inplace = deepcopy(self._rigs)
        kapture.rigs_recover_inplace(trajectories_inplace, rigs_inplace)
        trajectories_not_inplace = kapture.rigs_recover(self._trajectories_cams, self._rigs)
        self.assertTrue(equal_trajectories(trajectories_inplace, trajectories_not_inplace))
        # x = [pose.inverse().t_raw
        #      for _, _, pose in kapture.flatten(rigs_inplace, is_sorted=True)]
        # x = np.array(x)
        # import matplotlib.pyplot as plt
        # plt.plot(x[:, 0], x[:, 2], 'x--')
        # plt.show()

    def test_rig_sensors_ids(self):
        sensors_ids = self._trajectories_cams.sensors_ids
        self.assertEqual(len(sensors_ids), 2)
        self.assertIn('cam2', sensors_ids)


# RECORDS ##############################################################################################################
class TestRecords(unittest.TestCase):
    def test_init_camera(self):
        timestamp0, timestamp1 = 0, 1
        device_id0, device_id1 = 'cam0', 'cam1'
        record_cam0_image0 = 'cam0/image000.jpg'
        record_cam0_image1 = 'cam0/image001.jpg'
        record_cam1_image0 = 'cam1/image000.jpg'
        record_cam1_image1 = 'cam1/image001.jpg'
        # Test insertions
        records_camera = kapture.RecordsCamera()
        records_camera[timestamp0, device_id0] = record_cam0_image0
        kapture_data = kapture.Kapture(records_camera=records_camera)
        self.assertEqual(1, len(kapture_data.records_camera.keys()))
        self.assertEqual(1, len(kapture_data.records_camera.key_pairs()))
        self.assertEqual(record_cam0_image0, kapture_data.records_camera.data_list()[0])
        self.assertIn(timestamp0, kapture_data.records_camera)
        self.assertIn(device_id0, kapture_data.records_camera[timestamp0])
        self.assertIn((timestamp0, device_id0), kapture_data.records_camera)
        self.assertEqual(record_cam0_image0, kapture_data.records_camera[timestamp0, device_id0])
        records_camera[timestamp1, device_id0] = record_cam0_image1
        self.assertEqual(2, len(kapture_data.records_camera.keys()))
        self.assertEqual(2, len(kapture_data.records_camera.key_pairs()))
        self.assertEqual(2, len(kapture_data.records_camera.data_list()))
        self.assertEqual(record_cam0_image1, kapture_data.records_camera.data_list()[-1])
        kapture_data.records_camera[timestamp0][device_id1] = record_cam1_image0
        self.assertEqual(2, len(kapture_data.records_camera.keys()))
        self.assertEqual(3, len(kapture_data.records_camera.key_pairs()))
        self.assertEqual(3, len(kapture_data.records_camera.data_list()))
        self.assertEqual(record_cam0_image1, kapture_data.records_camera.data_list()[-1])
        records_camera[timestamp1][device_id1] = record_cam1_image1
        self.assertEqual(2, len(kapture_data.records_camera.keys()))
        self.assertEqual(4, len(kapture_data.records_camera.key_pairs()))
        self.assertEqual(4, len(kapture_data.records_camera.data_list()))
        self.assertEqual(record_cam1_image1, kapture_data.records_camera.data_list()[-1])
        self.assertEqual(record_cam0_image0, kapture_data.records_camera[timestamp0, device_id0])
        self.assertEqual(record_cam1_image0, kapture_data.records_camera[timestamp0, device_id1])
        self.assertEqual(record_cam0_image1, kapture_data.records_camera[timestamp1, device_id0])
        self.assertEqual(record_cam1_image1, kapture_data.records_camera[timestamp1, device_id1])

        self.assertNotIn((timestamp1, 'cam2'), kapture_data.records_camera)
        self.assertNotIn((2, device_id0), kapture_data.records_camera)
        self.assertEqual(kapture_data.records_camera.sensors_ids, {device_id0, device_id1})

        # Test deletion
        del kapture_data.records_camera[(timestamp0, device_id0)]
        self.assertEqual(2, len(kapture_data.records_camera.keys()))
        self.assertEqual(3, len(kapture_data.records_camera.key_pairs()))
        self.assertEqual(3, len(kapture_data.records_camera.data_list()))
        del kapture_data.records_camera[(timestamp0, device_id1)]
        self.assertEqual(1, len(kapture_data.records_camera.keys()))
        self.assertEqual(2, len(kapture_data.records_camera.key_pairs()))
        self.assertEqual(2, len(kapture_data.records_camera.data_list()))

    def test_init_lidar(self):
        records_lidar = kapture.RecordsLidar()
        self.assertIsNotNone(records_lidar, "Records Lidar created")

    def test_init_wifi(self):
        records_wifi = kapture.RecordsWifi()
        timestamp0, timestamp1 = 0, 1
        device_id0, device_id1 = 'AC01324954_WIFI', 'AC01324955_WIFI'
        bssid, ssid = '68:72:51:80:52:df', 'M1X_PicoM2'
        rssi = -33
        freq = 2417
        scan_time_start, scan_time_end = 1555398770280, 1555398770290
        # assign

        wifi_data = kapture.RecordWifi({bssid: kapture.RecordWifiSignal(ssid=ssid, rssi=rssi, frequency=freq,
                                                                        scan_time_start=scan_time_start,
                                                                        scan_time_end=scan_time_end)})
        records_wifi[timestamp0, device_id0] = wifi_data
        kapture_data = kapture.Kapture(records_wifi=records_wifi)
        self.assertEqual(1, len(kapture_data.records_wifi.keys()))
        self.assertEqual(1, len(kapture_data.records_wifi.key_pairs()))
        self.assertIn(timestamp0, kapture_data.records_wifi)
        self.assertIn(device_id0, kapture_data.records_wifi[timestamp0])
        self.assertIn((timestamp0, device_id0), kapture_data.records_wifi)
        self.assertEqual(wifi_data, kapture_data.records_wifi[timestamp0, device_id0])
        kapture_data.records_wifi[timestamp1, device_id1] = wifi_data
        self.assertEqual(2, len(kapture_data.records_wifi.keys()))
        self.assertEqual(2, len(kapture_data.records_wifi.key_pairs()))
        kapture_data.records_wifi[timestamp0][device_id1] = wifi_data
        self.assertEqual(2, len(kapture_data.records_wifi.keys()))
        self.assertEqual(3, len(kapture_data.records_wifi.key_pairs()))

    def test_init_gnss_epsg(self):
        gps_id1, gps_id2 = 'gps1', 'gps2'
        y, x, z = 51.388920, 30.099134, 15.0
        unix_ts = int(datetime(year=1986, month=4, day=26).timestamp())

        records_gnss = kapture.RecordsGnss()
        records_gnss[0, gps_id1] = kapture.RecordGnss(x + 0, y + 0, z + 0, unix_ts + 0, 9.)
        records_gnss[1, gps_id1] = kapture.RecordGnss(x + 1, y + 1, z + 1, unix_ts + 1, 2.)
        records_gnss[1, gps_id2] = kapture.RecordGnss(x + 2, y + 2, z + 2, unix_ts + 2, 2.)
        records_gnss[2] = {gps_id1: kapture.RecordGnss(x + 3, y + 3, z + 3, unix_ts + 3, 0.)}

        self.assertEqual(3, len(records_gnss))
        self.assertEqual(4, len(sorted(kapture.flatten(records_gnss))))
        self.assertIn((0, gps_id1), records_gnss)
        self.assertEqual(30.099134, records_gnss[0, gps_id1].x)
        self.assertEqual(51.388920, records_gnss[0, gps_id1].y)
        self.assertEqual(15., records_gnss[0, gps_id1].z)
        self.assertEqual(9., records_gnss[0, gps_id1].dop)

    def test_type_checking(self):
        records_camera = kapture.RecordsCamera()
        valid_ts, valid_id, valid_record = 0, 'cam0', 'cam0/image0.jpg'
        invalid_ts, invalid_id, invalid_record = '0', float(0), kapture.PoseTransform()
        self.assertRaises(TypeError, records_camera.__setitem__, (invalid_ts, valid_id), valid_record)
        self.assertRaises(TypeError, records_camera.__setitem__, (valid_ts, invalid_id), valid_record)
        self.assertRaises(TypeError, records_camera.__setitem__, (valid_ts, valid_id), invalid_record)
        self.assertRaises(TypeError, records_camera.__setitem__, (invalid_ts, invalid_id), invalid_record)

        self.assertRaises(TypeError, records_camera.__setitem__, invalid_ts, {valid_id: valid_record})
        self.assertRaises(TypeError, records_camera.__setitem__, valid_ts, {invalid_id: valid_record})
        self.assertRaises(TypeError, records_camera.__setitem__, valid_ts, {valid_id: invalid_record})
        self.assertRaises(TypeError, records_camera.__setitem__, invalid_ts, valid_record)

        self.assertRaises(TypeError, records_camera.__contains__, invalid_ts, valid_id)
        self.assertRaises(TypeError, records_camera.__contains__, valid_ts, invalid_id)
        self.assertRaises(TypeError, records_camera.__contains__, invalid_ts, invalid_id)

        self.assertRaises(TypeError, records_camera.__delitem__, invalid_ts)
        self.assertRaises(TypeError, records_camera.__delitem__, (valid_ts, invalid_id))


# Keypoints ############################################################################################################
class TestKeypoints(unittest.TestCase):
    def test_init_keypoints_sift(self):
        keypoints = kapture.Keypoints('SIFT', float, 4,
                                      ['a/a.jpg', 'b/b.jpg', 'c/c.jpg', 'c/c.jpg'])
        self.assertEqual('SIFT', keypoints.type_name)
        self.assertEqual(3, len(keypoints))
        self.assertIn('a/a.jpg', keypoints)

    def test_init_keypoints_unknown(self):
        keypoints = kapture.Keypoints('UNKNOWN', int, 64, ['a/a.jpg'])
        self.assertEqual('UNKNOWN', keypoints.type_name)


# Descriptors ##########################################################################################################
class TestDescriptors(unittest.TestCase):
    def test_init_descriptors_unknown(self):
        descriptors = kapture.Descriptors('R2D2', float, 64,
                                          'R2D2', 'L2',
                                          ['a/a.jpg', 'b/b.jpg', 'c/c.jpg', 'c/c.jpg'])
        self.assertEqual('R2D2', descriptors.type_name)
        self.assertEqual('R2D2', descriptors.keypoints_type)
        self.assertEqual('L2', descriptors.metric_type)
        self.assertEqual(3, len(descriptors))
        self.assertIn('a/a.jpg', descriptors)


# GlobalFeatures #######################################################################################################
class TestGlobalFeatures(unittest.TestCase):
    def test_init_global_features_unknown(self):
        global_features = kapture.GlobalFeatures('BOW', float, 4, 'CSI',
                                                 ['a/a.jpg', 'b/b.jpg', 'c/c.jpg', 'c/c.jpg'])
        self.assertEqual('BOW', global_features.type_name)
        self.assertEqual('CSI', global_features.metric_type)
        self.assertEqual(3, len(global_features))
        self.assertIn('a/a.jpg', global_features)


# Points3d #############################################################################################################
class TestPoints3d(unittest.TestCase):
    def test_empty(self):
        points3d = kapture.Points3d()
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (0, kapture.Points3d.XYZ_RGB))
        self.assertRaises(IndexError, points3d.__getitem__, (0, 0))
        self.assertFalse(points3d)

    def test_from_numpy_view(self):
        np_array = np.ones((10, 6))
        points3d = np_array.view(kapture.Points3d)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (10, kapture.Points3d.XYZ_RGB))
        self.assertTrue(points3d)

    def test_from_numpy_array(self):
        data = np.array([[0] * 6])
        points3d = kapture.Points3d(data)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (1, kapture.Points3d.XYZ_RGB))
        self.assertTrue(points3d)

        data = np.array([[0] * 3])
        points3d = kapture.Points3d(data)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (1, kapture.Points3d.XYZ_ONLY))
        self.assertTrue(points3d)

        data = np.arange(12).reshape((2, 6))
        points3d = kapture.Points3d(data)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (2, kapture.Points3d.XYZ_RGB))
        self.assertTrue(points3d)

        data = np.arange(15).reshape((5, 3))
        points3d = kapture.Points3d(data)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (5, kapture.Points3d.XYZ_ONLY))
        self.assertTrue(points3d)

        data = np.arange(16).reshape((2, 8))
        points3d = kapture.Points3d(data[:, 0:6])
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (2, kapture.Points3d.XYZ_RGB))
        self.assertTrue(points3d)

        data = list()
        data.append(list(range(1, 7)))
        points3d = kapture.Points3d(data)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (1, kapture.Points3d.XYZ_RGB))
        self.assertTrue(np.all(np.isclose(points3d.as_array(), data)))

        data = np.vstack([data, data])
        points3d = kapture.Points3d(data)
        self.assertIsInstance(points3d, kapture.Points3d)
        self.assertEqual(points3d.shape, (2, kapture.Points3d.XYZ_RGB))
        self.assertTrue(np.all(np.isclose(points3d.as_array(), data)))

    def test_from_numpy_array_invalid(self):
        data = np.arange(16).reshape((8, 2))
        with self.assertRaises(ValueError):
            kapture.Points3d(data)

        data = np.arange(16)
        with self.assertRaises(ValueError):
            kapture.Points3d(data)

    def test_slices(self):
        rows = 18
        float_array_values = np.array(range(0, rows * kapture.Points3d.XYZ_RGB), dtype=np.float64)
        np_points = np.reshape(float_array_values, (rows * 3, 2))
        self.assertRaises(ValueError, kapture.Points3d, np_points)

        np_points2 = np.reshape(np_points, (rows, kapture.Points3d.XYZ_RGB))
        # construct from numpy array
        points = kapture.Points3d(np_points2)
        # construct from a points3d
        points3d = kapture.Points3d(points)
        self.assertEqual(points, points3d)
        self.assertEqual(points3d[0, 0], 0.0)
        self.assertEqual(points3d[17, 5], 107.0)

        np.testing.assert_array_equal(points3d.as_array(), np_points2)
        np.testing.assert_array_equal(points3d[0, 0:4], float_array_values[0:4])
        np.testing.assert_array_equal(points3d[0, 4:], float_array_values[4:kapture.Points3d.XYZ_RGB])
        np.testing.assert_array_equal(points3d[0, :].as_array(), float_array_values[:kapture.Points3d.XYZ_RGB])

    def test_type_checking(self):
        self.assertRaises(TypeError, kapture.Points3d, (2, 3, 4))

        points3d = kapture.Points3d()
        self.assertEqual(points3d.dtype.name, 'float64')

        data = list()
        data.append(list(range(1, 7)))
        points3d = kapture.Points3d(data)
        self.assertEqual(points3d.dtype.name, 'float64')

        rows = 18
        float_list = np.array(range(0, rows * kapture.Points3d.XYZ_RGB))
        float_array = np.reshape(float_list, (rows, kapture.Points3d.XYZ_RGB))
        points3d = kapture.Points3d(float_array)
        self.assertEqual(points3d.dtype.name, 'float64')

        # test slicing
        val1 = points3d[:, 0:4]
        val2 = points3d[:]
        val3 = points3d[:, 0]
        val4 = points3d[0, :]
        val5 = points3d[0]
        val6 = points3d[0:2, :]
        val7 = points3d[0, 0]

        self.assertNotIsInstance(val1, kapture.Points3d)
        self.assertIsInstance(val2, kapture.Points3d)
        self.assertNotIsInstance(val3, kapture.Points3d)
        self.assertIsInstance(val4, kapture.Points3d)
        self.assertIsInstance(val5, kapture.Points3d)
        self.assertIsInstance(val6, kapture.Points3d)
        self.assertIsInstance(val7, float)


# OBSERVATIONS #########################################################################################################
class TestObservations(unittest.TestCase):
    def test_init_observations(self):
        observations = kapture.Observations({
            0: {'R2D2': [('a/a.jpg', 1), ('b/b.jpg', 2)],
                'D2NET': [('a/a.jpg', 0)]},
            1: {'R2D2': [('c/c.jpg', 1), ('c/c.jpg', 2)],
                'D2NET': [('c/c.jpg', 0), ('c/c.jpg', 1)]}
        })
        self.assertEqual(2, len(observations))
        self.assertEqual(7, observations.observations_number())
        self.assertTrue(0 in observations)
        self.assertTrue((0, 'R2D2') in observations)
        self.assertEqual(len(observations[0, 'R2D2']), 2)
        self.assertTrue((0, 'D2NET') in observations)
        self.assertEqual(len(observations[0, 'D2NET']), 1)

        self.assertTrue(1 in observations)
        self.assertTrue((1, 'R2D2') in observations)
        self.assertEqual(len(observations[1, 'R2D2']), 2)
        self.assertTrue((1, 'D2NET') in observations)
        self.assertEqual(len(observations[1, 'D2NET']), 2)

        observations.add(0, 'R2D2', 'c/c.jpg', 3)
        self.assertEqual(2, len(observations))
        self.assertEqual(8, observations.observations_number())
        self.assertEqual(len(observations[0, 'R2D2']), 3)

        observations.add(2, 'R2D2', 'a/a.jpg', 3)
        self.assertEqual(3, len(observations))
        self.assertEqual(9, observations.observations_number())
        self.assertTrue(2 in observations)
        self.assertTrue((2, 'R2D2') in observations)
        self.assertTrue((2, 'D2NET') not in observations)
        self.assertEqual(len(observations[2, 'R2D2']), 1)


# MATCHES ##############################################################################################################
class TestMatches(unittest.TestCase):
    def test_init_matches(self):
        matches = kapture.Matches(set([('bb', 'aa'), ('cc', 'dd')]))
        matches.normalize()

        self.assertFalse(('bb', 'aa') in matches)
        self.assertTrue(('aa', "bb") in matches)

        self.assertFalse(('dd', 'cc') in matches)
        self.assertTrue(('cc', 'dd') in matches)

        matches.add('ee', 'ff')
        matches.add('hh', 'gg')
        matches.normalize()

        self.assertFalse(('ff', 'ee') in matches)
        self.assertTrue(('ee', "ff") in matches)

        self.assertFalse(('hh', 'gg') in matches)
        self.assertTrue(('gg', 'hh') in matches)


# KAPTURE ##############################################################################################################
class TestKapture(unittest.TestCase):
    def test_init(self):
        lidar0 = kapture.Sensor(kapture.SensorType.lidar.name, [])
        cam0 = kapture.Sensor(kapture.SensorType.camera.name, [])
        sensors = kapture.Sensors()
        sensors['cam0'] = cam0
        kapture_data = kapture.Kapture(sensors=sensors)
        self.assertEqual(sensors, kapture_data.sensors)
        self.assertEqual(sensors['cam0'], kapture_data.sensors['cam0'])
        # assign
        sensors = kapture_data.sensors
        self.assertIsInstance(sensors, kapture.Sensors)
        kapture_data.sensors = sensors
        kapture_data.sensors['lidar0'] = lidar0

    def test_as_dict(self):
        kapture_data = kapture.Kapture()
        # test empty
        members = kapture_data.as_dict()
        self.assertEqual(members, {})
        members = kapture_data.as_dict(keep_none=True)
        self.assertEqual(len(members), 18)
        self.assertTrue(all(member is None for member in members.values()))

        # test sensors only
        kapture_data.sensors = kapture.Sensors({'cam0': kapture.Sensor(kapture.SensorType.camera.name, [])})
        members = kapture_data.as_dict()
        self.assertEqual(len(members), 1)
        self.assertEqual(members, {'sensors': kapture_data.sensors})
        members = kapture_data.as_dict(keep_none=True)
        self.assertEqual(len(members), 18)
        self.assertEqual(members['sensors'], kapture_data.sensors)
        self.assertTrue(all(member is None for name, member in members.items() if name != 'sensors'))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
from kapture.io.csv import kapture_from_dir
from kapture.algo.compare import equal_kapture
from kapture.io.records import TransferAction
from tools.kapture_import_7scenes import import_7scenes


class TestImport7scenes(unittest.TestCase):

    def setUp(self):
        d7scenes_sample_dirpath = path.abspath(path.join(path.dirname(__file__),  '../samples/7scenes'))
        self.d7scenes_rootpath = path.join(d7scenes_sample_dirpath, 'microsoft', 'stairs')
        self.kapture_gt_rootpath = path.join(d7scenes_sample_dirpath, 'kapture', 'stairs')
        self._tempdir = tempfile.TemporaryDirectory()

    def tearDown(self) -> None:
        self._tempdir.cleanup()

    def test_import_both(self):
        kapture_gt_dirpath = path.join(self.kapture_gt_rootpath, 'both')
        import_7scenes(d7scenes_path=self.d7scenes_rootpath,
                       kapture_dir_path=self._tempdir.name,
                       force_overwrite_existing=True,
                       images_import_method=TransferAction.copy)

        imported_kdata = kapture_from_dir(self._tempdir.name)
        expected_kdata = kapture_from_dir(kapture_gt_dirpath)
        self.assertTrue(equal_kapture(imported_kdata, expected_kdata))

    def test_import_mapping(self):
        kapture_gt_dirpath = path.join(self.kapture_gt_rootpath, 'mapping')
        import_7scenes(d7scenes_path=self.d7scenes_rootpath,
                       kapture_dir_path=self._tempdir.name,
                       force_overwrite_existing=True,
                       partition='mapping')

        imported_kdata = kapture_from_dir(self._tempdir.name)
        expected_kdata = kapture_from_dir(kapture_gt_dirpath)
        self.assertTrue(equal_kapture(imported_kdata, expected_kdata))

    def test_import_query(self):
        kapture_gt_dirpath = path.join(self.kapture_gt_rootpath, 'query')
        import_7scenes(d7scenes_path=self.d7scenes_rootpath,
                       kapture_dir_path=self._tempdir.name,
                       force_overwrite_existing=True,
                       partition='query')

        imported_kdata = kapture_from_dir(self._tempdir.name)
        expected_kdata = kapture_from_dir(kapture_gt_dirpath)
        self.assertTrue(equal_kapture(imported_kdata, expected_kdata))

    def test_import_sequence(self):
        kapture_gt_dirpath = path.join(self.kapture_gt_rootpath, 'seq-01')
        import_7scenes(d7scenes_path=self.d7scenes_rootpath + '/seq-01',
                       kapture_dir_path=self._tempdir.name,
                       force_overwrite_existing=True)

        imported_kdata = kapture_from_dir(self._tempdir.name)
        expected_kdata = kapture_from_dir(kapture_gt_dirpath)
        self.assertTrue(equal_kapture(imported_kdata, expected_kdata))


if __name__ == '__main__':
    unittest.main()
